@@ -20,8 +20,7 @@
  */
 
 
-#ifndef DLL_H
-#define DLL_H
+#pragma once
 
 /** The DLLManager class is able to load a module file by filename,
  * and locate its init_module symbol.
@@ -60,9 +59,12 @@ class CoreExport DLLManager : public classbase
 	 */
 	Module* CallInit();
 
+	/** Retrieves the value of the specified symbol.
+	 * @param name The name of the symbol to retrieve.
+	 * @return Either the value of the specified symbol or or NULL if it does not exist.
+	 */
+	void* GetSymbol(const char* name);
+
 	/** Get detailed version information from the module file */
 	std::string GetVersion();
 };
-
-#endif
-
