@@ -22,12 +22,10 @@
  */
 
 
-#ifndef USERS_H
-#define USERS_H
+#pragma once
 
 #include "socket.h"
 #include "inspsocket.h"
-#include "dns.h"
 #include "mode.h"
 #include "membership.h"
 
@@ -42,19 +40,6 @@ enum ClassTypes {
 	CC_NAMED = 2
 };
 
-/** RFC1459 channel modes
- */
-enum UserModes {
-	/** +s: Server notice mask */
-	UM_SNOMASK = 's' - 65,
-	/** +w: WALLOPS */
-	UM_WALLOPS = 'w' - 65,
-	/** +i: Invisible */
-	UM_INVISIBLE = 'i' - 65,
-	/** +o: Operator */
-	UM_OPERATOR = 'o' - 65
-};
-
 /** Registration state of a user, e.g.
  * have they sent USER, NICK, PASS yet?
  */
@@ -146,6 +131,15 @@ struct CoreExport ConnectClass : public refcountbase
 	 */
 	unsigned long limit;
 
+	/** If set to true, no user DNS lookups are to be performed
+	 */
+	bool resolvehostnames;
+
+	/**
+	 * If non-empty the server ports which this user has to be using
+	 */
+	insp::flat_set<int> ports;
+
 	/** Create a new connect class with no settings.
 	 */
 	ConnectClass(ConfigTag* tag, char type, const std::string& mask);
@@ -250,12 +244,40 @@ class CoreExport User : public Extensible
 	 */
 	std::string cachedip;
 
- public:
+	/** If set then the hostname which is displayed to users. */
+	std::string displayhost;
 
-	/** Hostname of connection.
-	 * This should be valid as per RFC1035.
+	/** The real hostname of this user. */
+	std::string realhost;
+
+	/** The real name of this user. */
+	std::string realname;
+
+	/** The user's mode list.
+	 * Much love to the STL for giving us an easy to use bitset, saving us RAM.
+	 * if (modes[modeid]) is set, then the mode is set.
+	 * For example, to work out if mode +i is set, we check the field
+	 * User::modes[invisiblemode->modeid] == true.
 	 */
-	std::string host;
+	std::bitset<ModeParser::MODEID_MAX> modes;
+
+ public:
+	/** To execute a function for each local neighbor of a user, inherit from this class and
+	 * pass an instance of it to User::ForEachNeighbor().
+	 */
+	class ForEachNeighborHandler
+	{
+	 public:
+		/** Method to execute for each local neighbor of a user.
+		 * Derived classes must implement this.
+		 * @param user Current neighbor
+		 */
+		virtual void Execute(LocalUser* user) = 0;
+	};
+
+	/** List of Memberships for this user
+	 */
+	typedef insp::intrusive_list<Membership> ChanList;
 
 	/** Time that the object was instantiated (used for TS calculation etc)
 	*/
@@ -267,10 +289,6 @@ class CoreExport User : public Extensible
 	 */
 	time_t signon;
 
-	/** Time that the connection last sent a message, used to calculate idle time
-	 */
-	time_t idle_lastmsg;
-
 	/** Client address that the user is connected from.
 	 * Do not modify this value directly, use SetClientIP() to change it.
 	 * Port is not valid for remote users.
@@ -293,27 +311,6 @@ class CoreExport User : public Extensible
 	 */
 	std::string ident;
 
-	/** The host displayed to non-opers (used for cloaking etc).
-	 * This usually matches the value of User::host.
-	 */
-	std::string dhost;
-
-	/** The users full name (GECOS).
-	 */
-	std::string fullname;
-
-	/** The user's mode list.
-	 * NOT a null terminated string.
-	 * Also NOT an array.
-	 * Much love to the STL for giving us an easy to use bitset, saving us RAM.
-	 * if (modes[modeletter-65]) is set, then the mode is
-	 * set, for example, to work out if mode +s is set, we  check the field
-	 * User::modes['s'-65] != 0.
-	 * The following RFC characters o, w, s, i have constants defined via an
-	 * enum, such as UM_SERVERNOTICE and UM_OPETATOR.
-	 */
-	std::bitset<64> modes;
-
 	/** What snomasks are set on this user.
 	 * This functions the same as the above modes.
 	 */
@@ -321,11 +318,11 @@ class CoreExport User : public Extensible
 
 	/** Channels this user is on
 	 */
-	UserChanList chans;
+	ChanList chans;
 
 	/** The server the user is connected to.
 	 */
-	const std::string server;
+	Server* server;
 
 	/** The user's away message.
 	 * If this string is empty, the user is not marked as away.
@@ -333,7 +330,7 @@ class CoreExport User : public Extensible
 	std::string awaymsg;
 
 	/** Time the user last went away.
-	 * This is ONLY RELIABLE if user IS_AWAY()!
+	 * This is ONLY RELIABLE if user IsAway()!
 	 */
 	time_t awaytime;
 
@@ -347,16 +344,6 @@ class CoreExport User : public Extensible
 	 */
 	unsigned int registered:3;
 
-	/** True when DNS lookups are completed.
-	 * The UserResolver classes res_forward and res_reverse will
-	 * set this value once they complete.
-	 */
-	unsigned int dns_done:1;
-
-	/** Whether or not to send an snotice about this user's quitting
-	 */
-	unsigned int quietquit:1;
-
 	/** If this is set to true, then all socket operations for the user
 	 * are dropped into the bit-bucket.
 	 * This value is set by QuitUser, and is not needed seperately from that call.
@@ -364,27 +351,27 @@ class CoreExport User : public Extensible
 	 */
 	unsigned int quitting:1;
 
-	/** Recursion fix: user is out of SendQ and will be quit as soon as possible.
-	 * This can't be handled normally because QuitUser itself calls Write on other
-	 * users, which could trigger their SendQ to overrun.
-	 */
-	unsigned int quitting_sendq:1;
-
-	/** This is true if the user matched an exception (E:Line). It is used to save time on ban checks.
-	 */
-	unsigned int exempt:1;
-
-	/** has the user responded to their previous ping?
-	 */
-	unsigned int lastping:1;
-
 	/** What type of user is this? */
-	const unsigned int usertype:2;
+	const UserType usertype:2;
 
 	/** Get client IP string from sockaddr, using static internal buffer
 	 * @return The IP string
 	 */
-	const char* GetIPString();
+	const std::string& GetIPString();
+
+	/** Retrieves this user's hostname.
+	 * @param uncloak If true then return the real host; otherwise, the display host.
+	 */
+	const std::string& GetHost(bool uncloak) const;
+
+	/** Retrieves this user's displayed hostname. */
+	const std::string& GetDisplayedHost() const;
+
+	/** Retrieves this user's real hostname. */
+	const std::string& GetRealHost() const;
+
+	/** Retrieves this user's real name. */
+	const std::string& GetRealName() const;
 
 	/** Get CIDR mask, using default range, for this user
 	 */
@@ -393,20 +380,14 @@ class CoreExport User : public Extensible
 	/** Sets the client IP for this user
 	 * @return true if the conversion was successful
 	 */
-	virtual bool SetClientIP(const char* sip, bool recheck_eline = true);
+	virtual bool SetClientIP(const std::string& address);
 
-	virtual void SetClientIP(const irc::sockets::sockaddrs& sa, bool recheck_eline = true);
+	virtual void SetClientIP(const irc::sockets::sockaddrs& sa);
 
 	/** Constructor
 	 * @throw CoreException if the UID allocated to the user already exists
 	 */
-	User(const std::string &uid, const std::string& srv, int objtype);
-
-	/** Check if the user matches a G or K line, and disconnect them if they do.
-	 * @param doZline True if ZLines should be checked (if IP has changed since initial connect)
-	 * Returns true if the user matched a ban, false else.
-	 */
-	bool CheckLines(bool doZline = false);
+	User(const std::string& uid, Server* srv, UserType objtype);
 
 	/** Returns the full displayed host of the user
 	 * This member function returns the hostname of the user as seen by other users
@@ -428,18 +409,17 @@ class CoreExport User : public Extensible
 	 */
 	void InvalidateCache();
 
-	/** Create a displayable mode string for this users snomasks
-	 * @return The notice mask character sequence
+	/** Returns whether this user is currently away or not. If true,
+	 * further information can be found in User::awaymsg and User::awaytime
+	 * @return True if the user is away, false otherwise
 	 */
-	const char* FormatNoticeMasks();
+	bool IsAway() const { return (!awaymsg.empty()); }
 
-	/** Process a snomask modifier string, e.g. +abc-de
-	 * @param sm A sequence of notice mask characters
-	 * @return The cleaned mode sequence which can be output,
-	 * e.g. in the above example if masks c and e are not
-	 * valid, this function will return +ab-d
+	/** Returns whether this user is an oper or not. If true,
+	 * oper information can be obtained from User::oper
+	 * @return True if the user is an oper, false otherwise
 	 */
-	std::string ProcessNoticeMasks(const char *sm);
+	bool IsOper() const { return oper; }
 
 	/** Returns true if a notice mask is set
 	 * @param sm A notice mask character to check
@@ -447,28 +427,28 @@ class CoreExport User : public Extensible
 	 */
 	bool IsNoticeMaskSet(unsigned char sm);
 
-	/** Changed a specific notice mask value
-	 * @param sm The server notice mask to change
-	 * @param value An on/off value for this mask
+	/** Get the mode letters of modes set on the user as a string.
+	 * @param includeparams True to get the parameters of the modes as well. Defaults to false.
+	 * @return Mode letters of modes set on the user and optionally the parameters of those modes, if any.
+	 * The returned string always begins with a '+' character. If the user has no modes set, "+" is returned.
 	 */
-	void SetNoticeMask(unsigned char sm, bool value);
-
-	/** Create a displayable mode string for this users umodes
-	 * @param showparameters The mode string
-	 */
-	const char* FormatModes(bool showparameters = false);
+	std::string GetModeLetters(bool includeparams = false) const;
 
 	/** Returns true if a specific mode is set
 	 * @param m The user mode
 	 * @return True if the mode is set
 	 */
-	bool IsModeSet(unsigned char m);
+	bool IsModeSet(unsigned char m) const;
+	bool IsModeSet(const ModeHandler* mh) const;
+	bool IsModeSet(const ModeHandler& mh) const { return IsModeSet(&mh); }
+	bool IsModeSet(UserModeReference& moderef) const;
 
 	/** Set a specific usermode to on or off
-	 * @param m The user mode
+	 * @param mh The user mode
 	 * @param value On or off setting of the mode
 	 */
-	void SetMode(unsigned char m, bool value);
+	void SetMode(ModeHandler* mh, bool value);
+	void SetMode(ModeHandler& mh, bool value) { SetMode(&mh, value); }
 
 	/** Returns true or false for if a user can execute a privilaged oper command.
 	 * This is done by looking up their oper type from User::oper, then referencing
@@ -476,32 +456,24 @@ class CoreExport User : public Extensible
 	 * @param command A command (should be all CAPS)
 	 * @return True if this user can execute the command
 	 */
-	virtual bool HasPermission(const std::string &command);
+	virtual bool HasCommandPermission(const std::string& command);
 
 	/** Returns true if a user has a given permission.
 	 * This is used to check whether or not users may perform certain actions which admins may not wish to give to
 	 * all operators, yet are not commands. An example might be oper override, mass messaging (/notice $*), etc.
 	 *
 	 * @param privstr The priv to chec, e.g. "users/override/topic". These are loaded free-form from the config file.
-	 * @param noisy If set to true, the user is notified that they do not have the specified permission where applicable. If false, no notification is sent.
 	 * @return True if this user has the permission in question.
 	 */
-	virtual bool HasPrivPermission(const std::string &privstr, bool noisy = false);
+	virtual bool HasPrivPermission(const std::string& privstr);
 
 	/** Returns true or false if a user can set a privileged user or channel mode.
 	 * This is done by looking up their oper type from User::oper, then referencing
 	 * this to their oper classes, and checking the modes they can set.
-	 * @param mode The mode the check
-	 * @param type ModeType (MODETYPE_CHANNEL or MODETYPE_USER).
+	 * @param mh Mode to check
 	 * @return True if the user can set or unset this mode.
 	 */
-	virtual bool HasModePermission(unsigned char mode, ModeType type);
-
-	/** Creates a wildcard host.
-	 * Takes a buffer to use and fills the given buffer with the host in the format *!*\@hostname
-	 * @return The wildcarded hostname in *!*\@host form
-	 */
-	char* MakeWildHost();
+	virtual bool HasModePermission(const ModeHandler* mh) const;
 
 	/** Creates a usermask with real host.
 	 * Takes a buffer to use and fills the given buffer with the hostmask in the format user\@host
@@ -515,122 +487,145 @@ class CoreExport User : public Extensible
 	 */
 	const std::string& MakeHostIP();
 
-	/** Add the user to WHOWAS system
-	 */
-	void AddToWhoWas();
-
 	/** Oper up the user using the given opertype.
 	 * This will also give the +o usermode.
 	 */
 	void Oper(OperInfo* info);
 
-	/** Force a nickname change.
-	 * If the nickname change fails (for example, because the nick in question
-	 * already exists) this function will return false, and you must then either
-	 * output an error message, or quit the user for nickname collision.
-	 * @param newnick The nickname to change to
-	 * @return True if the nickchange was successful.
-	 */
-	inline bool ForceNickChange(const char* newnick) { return ChangeNick(newnick, true); }
-
 	/** Oper down.
 	 * This will clear the +o usermode and unset the user's oper type
 	 */
 	void UnOper();
 
-	/** Write text to this user, appending CR/LF. Works on local users only.
-	 * @param text A std::string to send to the user
+	/** Sends a server notice to this user.
+	 * @param text The contents of the message to send.
 	 */
-	virtual void Write(const std::string &text);
+	void WriteNotice(const std::string& text);
 
-	/** Write text to this user, appending CR/LF.
-	 * Works on local users only.
-	 * @param text The format string for text to send to the user
-	 * @param ... POD-type format arguments
+	/** Send a NOTICE message from the local server to the user.
+	 * @param text Text to send
 	 */
-	virtual void Write(const char *text, ...) CUSTOM_PRINTF(2, 3);
+	virtual void WriteRemoteNotice(const std::string& text);
 
-	/** Write text to this user, appending CR/LF and prepending :server.name
-	 * Works on local users only.
-	 * @param text A std::string to send to the user
-	 */
-	void WriteServ(const std::string& text);
+	virtual void WriteRemoteNumeric(const Numeric::Numeric& numeric);
 
-	/** Write text to this user, appending CR/LF and prepending :server.name
-	 * Works on local users only.
-	 * @param text The format string for text to send to the user
-	 * @param ... POD-type format arguments
-	 */
-	void WriteServ(const char* text, ...) CUSTOM_PRINTF(2, 3);
+	template <typename T1>
+	void WriteRemoteNumeric(unsigned int numeric, T1 p1)
+	{
+		Numeric::Numeric n(numeric);
+		n.push(p1);
+		WriteRemoteNumeric(n);
+	}
 
-	void WriteNumeric(unsigned int numeric, const char* text, ...) CUSTOM_PRINTF(3, 4);
+	template <typename T1, typename T2>
+	void WriteRemoteNumeric(unsigned int numeric, T1 p1, T2 p2)
+	{
+		Numeric::Numeric n(numeric);
+		n.push(p1);
+		n.push(p2);
+		WriteRemoteNumeric(n);
+	}
 
-	void WriteNumeric(unsigned int numeric, const std::string &text);
+	template <typename T1, typename T2, typename T3>
+	void WriteRemoteNumeric(unsigned int numeric, T1 p1, T2 p2, T3 p3)
+	{
+		Numeric::Numeric n(numeric);
+		n.push(p1);
+		n.push(p2);
+		n.push(p3);
+		WriteRemoteNumeric(n);
+	}
 
-	/** Write text to this user, appending CR/LF and prepending :nick!user\@host of the user provided in the first parameter.
-	 * @param user The user to prepend the :nick!user\@host of
-	 * @param text A std::string to send to the user
-	 */
-	void WriteFrom(User *user, const std::string &text);
+	template <typename T1, typename T2, typename T3, typename T4>
+	void WriteRemoteNumeric(unsigned int numeric, T1 p1, T2 p2, T3 p3, T4 p4)
+	{
+		Numeric::Numeric n(numeric);
+		n.push(p1);
+		n.push(p2);
+		n.push(p3);
+		n.push(p4);
+		WriteRemoteNumeric(n);
+	}
 
-	/** Write text to this user, appending CR/LF and prepending :nick!user\@host of the user provided in the first parameter.
-	 * @param user The user to prepend the :nick!user\@host of
-	 * @param text The format string for text to send to the user
-	 * @param ... POD-type format arguments
-	 */
-	void WriteFrom(User *user, const char* text, ...) CUSTOM_PRINTF(3, 4);
+	template <typename T1, typename T2, typename T3, typename T4, typename T5>
+	void WriteRemoteNumeric(unsigned int numeric, T1 p1, T2 p2, T3 p3, T4 p4, T5 p5)
+	{
+		Numeric::Numeric n(numeric);
+		n.push(p1);
+		n.push(p2);
+		n.push(p3);
+		n.push(p4);
+		n.push(p5);
+		WriteRemoteNumeric(n);
+	}
 
-	/** Write text to the user provided in the first parameter, appending CR/LF, and prepending THIS user's :nick!user\@host.
-	 * @param dest The user to route the message to
-	 * @param data A std::string to send to the user
-	 */
-	void WriteTo(User *dest, const std::string &data);
+	void WriteNumeric(const Numeric::Numeric& numeric);
 
-	/** Write text to the user provided in the first parameter, appending CR/LF, and prepending THIS user's :nick!user\@host.
-	 * @param dest The user to route the message to
-	 * @param data The format string for text to send to the user
-	 * @param ... POD-type format arguments
-	 */
-	void WriteTo(User *dest, const char *data, ...) CUSTOM_PRINTF(3, 4);
+	template <typename T1>
+	void WriteNumeric(unsigned int numeric, T1 p1)
+	{
+		Numeric::Numeric n(numeric);
+		n.push(p1);
+		WriteNumeric(n);
+	}
+
+	template <typename T1, typename T2>
+	void WriteNumeric(unsigned int numeric, T1 p1, T2 p2)
+	{
+		Numeric::Numeric n(numeric);
+		n.push(p1);
+		n.push(p2);
+		WriteNumeric(n);
+	}
+
+	template <typename T1, typename T2, typename T3>
+	void WriteNumeric(unsigned int numeric, T1 p1, T2 p2, T3 p3)
+	{
+		Numeric::Numeric n(numeric);
+		n.push(p1);
+		n.push(p2);
+		n.push(p3);
+		WriteNumeric(n);
+	}
+
+	template <typename T1, typename T2, typename T3, typename T4>
+	void WriteNumeric(unsigned int numeric, T1 p1, T2 p2, T3 p3, T4 p4)
+	{
+		Numeric::Numeric n(numeric);
+		n.push(p1);
+		n.push(p2);
+		n.push(p3);
+		n.push(p4);
+		WriteNumeric(n);
+	}
+
+	template <typename T1, typename T2, typename T3, typename T4, typename T5>
+	void WriteNumeric(unsigned int numeric, T1 p1, T2 p2, T3 p3, T4 p4, T5 p5)
+	{
+		Numeric::Numeric n(numeric);
+		n.push(p1);
+		n.push(p2);
+		n.push(p3);
+		n.push(p4);
+		n.push(p5);
+		WriteNumeric(n);
+	}
 
 	/** Write to all users that can see this user (including this user in the list if include_self is true), appending CR/LF
-	 * @param line A std::string to send to the users
+	 * @param protoev Protocol event to send, may contain any number of messages.
 	 * @param include_self Should the message be sent back to the author?
 	 */
-	void WriteCommonRaw(const std::string &line, bool include_self = true);
+	void WriteCommonRaw(ClientProtocol::Event& protoev, bool include_self = true);
 
-	/** Write to all users that can see this user (including this user in the list), appending CR/LF
-	 * @param text The format string for text to send to the users
-	 * @param ... POD-type format arguments
+	/** Execute a function once for each local neighbor of this user. By default, the neighbors of a user are the users
+	 * who have at least one common channel with the user. Modules are allowed to alter the set of neighbors freely.
+	 * This function is used for example to send something conditionally to neighbors, or to send different messages
+	 * to different users depending on their oper status.
+	 * @param handler Function object to call, inherited from ForEachNeighborHandler.
+	 * @param include_self True to include this user in the set of neighbors, false otherwise.
+	 * Modules may override this. Has no effect if this user is not local.
 	 */
-	void WriteCommon(const char* text, ...) CUSTOM_PRINTF(2, 3);
-
-	/** Write to all users that can see this user (not including this user in the list), appending CR/LF
-	 * @param text The format string for text to send to the users
-	 * @param ... POD-type format arguments
-	 */
-	void WriteCommonExcept(const char* text, ...) CUSTOM_PRINTF(2, 3);
-
-	/** Write a quit message to all common users, as in User::WriteCommonExcept but with a specific
-	 * quit message for opers only.
-	 * @param normal_text Normal user quit message
-	 * @param oper_text Oper only quit message
-	 */
-	void WriteCommonQuit(const std::string &normal_text, const std::string &oper_text);
-
-	/** Dump text to a user target, splitting it appropriately to fit
-	 * @param LinePrefix text to prefix each complete line with
-	 * @param TextStream the text to send to the user
-	 */
-	void SendText(const std::string &LinePrefix, std::stringstream &TextStream);
-
-	/** Write to the user, routing the line if the user is remote.
-	 */
-	virtual void SendText(const std::string& line) = 0;
-
-	/** Write to the user, routing the line if the user is remote.
-	 */
-	void SendText(const char* text, ...) CUSTOM_PRINTF(2, 3);
+	void ForEachNeighbor(ForEachNeighborHandler& handler, bool include_self = true);
 
 	/** Return true if the user shares at least one channel with another user
 	 * @param other The other user to compare the channel list against
@@ -638,96 +633,72 @@ class CoreExport User : public Extensible
 	 */
 	bool SharesChannelWith(User *other);
 
-	/** Send fake quit/join messages for host or ident cycle.
-	 * Run this after the item in question has changed.
-	 * You should not need to use this function, call ChangeDisplayedHost instead
-	 *
-	 * @param quitline The entire QUIT line, including the source using the old value
+	/** Change the displayed hostname of this user.
+	 * @param host The new displayed hostname of this user.
+	 * @return True if the hostname was changed successfully; otherwise, false.
 	 */
-	void DoHostCycle(const std::string &quitline);
+	bool ChangeDisplayedHost(const std::string& host);
 
-	/** Change the displayed host of a user.
-	 * ALWAYS use this function, rather than writing User::dhost directly,
-	 * as this triggers module events allowing the change to be syncronized to
-	 * remote servers. This will also emulate a QUIT and rejoin (where configured)
-	 * before setting their host field.
-	 * @param host The new hostname to set
-	 * @return True if the change succeeded, false if it didn't
+	/** Change the real hostname of this user.
+	 * @param host The new real hostname of this user.
+	 * @param resetdisplay Whether to reset the display host to this value.
 	 */
-	bool ChangeDisplayedHost(const char* host);
+	void ChangeRealHost(const std::string& host, bool resetdisplay);
 
 	/** Change the ident (username) of a user.
 	 * ALWAYS use this function, rather than writing User::ident directly,
-	 * as this correctly causes the user to seem to quit (where configured)
-	 * before setting their ident field.
+	 * as this triggers module events allowing the change to be syncronized to
+	 * remote servers.
 	 * @param newident The new ident to set
 	 * @return True if the change succeeded, false if it didn't
 	 */
-	bool ChangeIdent(const char* newident);
+	bool ChangeIdent(const std::string& newident);
 
 	/** Change a users realname field.
-	 * ALWAYS use this function, rather than writing User::fullname directly,
-	 * as this triggers module events allowing the change to be syncronized to
-	 * remote servers.
-	 * @param gecos The user's new realname
+	 * @param real The user's new real name
 	 * @return True if the change succeeded, false if otherwise
 	 */
-	bool ChangeName(const char* gecos);
+	bool ChangeRealName(const std::string& real);
 
 	/** Change a user's nick
-	 * @param newnick The new nick
-	 * @param force True if the change is being forced (should not be blocked by modes like +N)
+	 * @param newnick The new nick. If equal to the users uuid, the nick change always succeeds.
+	 * @param newts The time at which this nick change happened.
 	 * @return True if the change succeeded
 	 */
-	bool ChangeNick(const std::string& newnick, bool force = false);
-
-	/** Send a command to all local users from this user
-	 * The command given must be able to send text with the
-	 * first parameter as a servermask (e.g. $*), so basically
-	 * you should use PRIVMSG or NOTICE.
-	 * @param command the command to send
-	 * @param text The text format string to send
-	 * @param ... Format arguments
-	 */
-	void SendAll(const char* command, const char* text, ...) CUSTOM_PRINTF(3, 4);
-
-	/** Compile a channel list for this user.  Used internally by WHOIS
-	 * @param source The user to prepare the channel list for
-	 * @param spy Whether to return the spy channel list rather than the normal one
-	 * @return This user's channel list
-	 */
-	std::string ChannelList(User* source, bool spy);
-
-	/** Split the channel list in cl which came from dest, and spool it to this user
-	 * Used internally by WHOIS
-	 * @param dest The user the original channel list came from
-	 * @param cl The  channel list as a string obtained from User::ChannelList()
-	 */
-	void SplitChanList(User* dest, const std::string &cl);
+	bool ChangeNick(const std::string& newnick, time_t newts = 0);
 
 	/** Remove this user from all channels they are on, and delete any that are now empty.
 	 * This is used by QUIT, and will not send part messages!
 	 */
 	void PurgeEmptyChannels();
 
-	/** Get the connect class which this user belongs to. NULL for remote users.
-	 * @return A pointer to this user's connect class.
-	 */
-	virtual ConnectClass* GetClass();
-
 	/** Default destructor
 	 */
 	virtual ~User();
-	virtual CullResult cull();
+	CullResult cull() CXX11_OVERRIDE;
+
+	/** @copydoc Serializable::Deserialize. */
+	bool Deserialize(Data& data) CXX11_OVERRIDE;
+
+	/** @copydoc Serializable::Deserialize. */
+	bool Serialize(Serializable::Data& data) CXX11_OVERRIDE;
 };
 
 class CoreExport UserIOHandler : public StreamSocket
 {
+ private:
+	 size_t checked_until;
  public:
 	LocalUser* const user;
-	UserIOHandler(LocalUser* me) : user(me) {}
-	void OnDataReady();
-	void OnError(BufferedSocketError error);
+	UserIOHandler(LocalUser* me)
+		: StreamSocket(StreamSocket::SS_USER)
+		, checked_until(0)
+		, user(me)
+	{
+	}
+	void OnDataReady() CXX11_OVERRIDE;
+	bool OnSetEndPoint(const irc::sockets::sockaddrs& local, const irc::sockets::sockaddrs& remote) CXX11_OVERRIDE;
+	void OnError(BufferedSocketError error) CXX11_OVERRIDE;
 
 	/** Adds to the user's write buffer.
 	 * You may add any amount of text up to this users sendq value, if you exceed the
@@ -735,21 +706,44 @@ class CoreExport UserIOHandler : public StreamSocket
 	 * @param data The data to add to the write buffer
 	 */
 	void AddWriteBuf(const std::string &data);
+
+	/** Swaps the internals of this UserIOHandler with another one.
+	 * @param other A UserIOHandler to swap internals with.
+	 */
+	void SwapInternals(UserIOHandler& other);
 };
 
 typedef unsigned int already_sent_t;
 
-class CoreExport LocalUser : public User, public InviteBase
+class CoreExport LocalUser : public User, public insp::intrusive_list_node<LocalUser>
 {
+	/** Add a serialized message to the send queue of the user.
+	 * @param serialized Bytes to add.
+	 */
+	void Write(const ClientProtocol::SerializedMessage& serialized);
+
+	/** Send a protocol event to the user, consisting of one or more messages.
+	 * @param protoev Event to send, may contain any number of messages.
+	 * @param msglist Message list used temporarily internally to pass to hooks and store messages
+	 * before Write().
+	 */
+	void Send(ClientProtocol::Event& protoev, ClientProtocol::MessageList& msglist);
+
+	/** Message list, can be passed to the two parameter Send().
+	 */
+	static ClientProtocol::MessageList sendmsglist;
+
  public:
 	LocalUser(int fd, irc::sockets::sockaddrs* client, irc::sockets::sockaddrs* server);
-	CullResult cull();
+	LocalUser(int fd, const std::string& uuid, Serializable::Data& data);
+
+	CullResult cull() CXX11_OVERRIDE;
 
 	UserIOHandler eh;
 
-	/** Position in UserManager::local_users
+	/** Serializer to use when communicating with the user
 	 */
-	LocalUserList::iterator localuseriter;
+	ClientProtocol::Serializer* serializer;
 
 	/** Stats counter for bytes inbound
 	 */
@@ -777,45 +771,55 @@ class CoreExport LocalUser : public User, public InviteBase
 	 */
 	reference<ConnectClass> MyClass;
 
-	ConnectClass* GetClass();
+	/** Get the connect class which this user belongs to.
+	 * @return A pointer to this user's connect class.
+	 */
+	ConnectClass* GetClass() const { return MyClass; }
 
 	/** Call this method to find the matching \<connect> for a user, and to check them against it.
 	 */
-	void CheckClass();
+	void CheckClass(bool clone_count = true);
 
 	/** Server address and port that this user is connected to.
 	 */
 	irc::sockets::sockaddrs server_sa;
 
-	/**
-	 * @return The port number of this user.
+	/** Recursion fix: user is out of SendQ and will be quit as soon as possible.
+	 * This can't be handled normally because QuitUser itself calls Write on other
+	 * users, which could trigger their SendQ to overrun.
 	 */
-	int GetServerPort();
+	unsigned int quitting_sendq:1;
 
-	/** Used by PING checking code
+	/** has the user responded to their previous ping?
 	 */
-	time_t nping;
+	unsigned int lastping:1;
+
+	/** This is true if the user matched an exception (E-line). It is used to save time on ban checks.
+	 */
+	unsigned int exempt:1;
+
+	/** The time at which this user should be pinged next. */
+	time_t nextping;
+
+	/** Time that the connection last sent a message, used to calculate idle time
+	 */
+	time_t idle_lastmsg;
 
 	/** This value contains how far into the penalty threshold the user is.
 	 * This is used either to enable fake lag or for excess flood quits
 	 */
 	unsigned int CommandFloodPenalty;
 
-	static already_sent_t already_sent_id;
 	already_sent_t already_sent;
 
-	/** Stored reverse lookup from res_forward. Should not be used after resolution.
+	/** Check if the user matches a G- or K-line, and disconnect them if they do.
+	 * @param doZline True if Z-lines should be checked (if IP has changed since initial connect)
+	 * Returns true if the user matched a ban, false else.
 	 */
-	std::string stored_host;
-
-	/** Starts a DNS lookup of the user's IP.
-	 * This will cause two UserResolver classes to be instantiated.
-	 * When complete, these objects set User::dns_done to true.
-	 */
-	void StartDNSLookup();
+	bool CheckLines(bool doZline = false);
 
 	/** Use this method to fully connect a user.
-	 * This will send the message of the day, check G/K/E lines, etc.
+	 * This will send the message of the day, check G/K/E-lines, etc.
 	 */
 	void FullConnect();
 
@@ -825,39 +829,15 @@ class CoreExport LocalUser : public User, public InviteBase
 	 */
 	void SetClass(const std::string &explicit_name = "");
 
-	bool SetClientIP(const char* sip, bool recheck_eline = true);
+	bool SetClientIP(const std::string& address) CXX11_OVERRIDE;
 
-	void SetClientIP(const irc::sockets::sockaddrs& sa, bool recheck_eline = true);
+	void SetClientIP(const irc::sockets::sockaddrs& sa) CXX11_OVERRIDE;
 
-	void SendText(const std::string& line);
-	void Write(const std::string& text);
-	void Write(const char*, ...) CUSTOM_PRINTF(2, 3);
-
-	/** Returns the list of channels this user has been invited to but has not yet joined.
-	 * @return A list of channels the user is invited to
+	/** Send a NOTICE message from the local server to the user.
+	 * The message will be sent even if the user is connected to a remote server.
+	 * @param text Text to send
 	 */
-	InviteList& GetInviteList();
-
-	/** Returns true if a user is invited to a channel.
-	 * @param channel A channel name to look up
-	 * @return True if the user is invited to the given channel
-	 */
-	bool IsInvited(const irc::string &channel);
-
-	/** Adds a channel to a users invite list (invites them to a channel)
-	 * @param channel A channel name to add
-	 * @param timeout When the invite should expire (0 == never)
-	 */
-	void InviteTo(const irc::string &channel, time_t timeout);
-
-	/** Removes a channel from a users invite list.
-	 * This member function is called on successfully joining an invite only channel
-	 * to which the user has previously been invited, to clear the invitation.
-	 * @param channel The channel to remove the invite to
-	 */
-	void RemoveInvite(const irc::string &channel);
-
-	void RemoveExpiredInvites();
+	void WriteRemoteNotice(const std::string& text) CXX11_OVERRIDE;
 
 	/** Returns true or false for if a user can execute a privilaged oper command.
 	 * This is done by looking up their oper type from User::oper, then referencing
@@ -865,103 +845,108 @@ class CoreExport LocalUser : public User, public InviteBase
 	 * @param command A command (should be all CAPS)
 	 * @return True if this user can execute the command
 	 */
-	bool HasPermission(const std::string &command);
+	bool HasCommandPermission(const std::string& command) CXX11_OVERRIDE;
 
 	/** Returns true if a user has a given permission.
 	 * This is used to check whether or not users may perform certain actions which admins may not wish to give to
 	 * all operators, yet are not commands. An example might be oper override, mass messaging (/notice $*), etc.
 	 *
 	 * @param privstr The priv to chec, e.g. "users/override/topic". These are loaded free-form from the config file.
-	 * @param noisy If set to true, the user is notified that they do not have the specified permission where applicable. If false, no notification is sent.
 	 * @return True if this user has the permission in question.
 	 */
-	bool HasPrivPermission(const std::string &privstr, bool noisy = false);
+	bool HasPrivPermission(const std::string& privstr) CXX11_OVERRIDE;
 
 	/** Returns true or false if a user can set a privileged user or channel mode.
 	 * This is done by looking up their oper type from User::oper, then referencing
 	 * this to their oper classes, and checking the modes they can set.
-	 * @param mode The mode the check
-	 * @param type ModeType (MODETYPE_CHANNEL or MODETYPE_USER).
+	 * @param mh Mode to check
 	 * @return True if the user can set or unset this mode.
 	 */
-	bool HasModePermission(unsigned char mode, ModeType type);
+	bool HasModePermission(const ModeHandler* mh) const CXX11_OVERRIDE;
+
+	/** Change nick to uuid, unset REG_NICK and send a nickname overruled numeric.
+	 * This is called when another user (either local or remote) needs the nick of this user and this user
+	 * isn't registered.
+	 */
+	void OverruleNick();
+
+	/** Send a protocol event to the user, consisting of one or more messages.
+	 * @param protoev Event to send, may contain any number of messages.
+	 */
+	void Send(ClientProtocol::Event& protoev);
+
+	/** Send a single message to the user.
+	 * @param protoevprov Protocol event provider.
+	 * @param msg Message to send.
+	 */
+	void Send(ClientProtocol::EventProvider& protoevprov, ClientProtocol::Message& msg);
+
+	/** @copydoc Serializable::Deserialize. */
+	bool Deserialize(Data& data) CXX11_OVERRIDE;
+
+	/** @copydoc Serializable::Deserialize. */
+	bool Serialize(Serializable::Data& data) CXX11_OVERRIDE;
 };
 
-class CoreExport RemoteUser : public User
+class RemoteUser : public User
 {
  public:
-	RemoteUser(const std::string& uid, const std::string& srv) : User(uid, srv, USERTYPE_REMOTE)
+	RemoteUser(const std::string& uid, Server* srv) : User(uid, srv, USERTYPE_REMOTE)
 	{
 	}
-	virtual void SendText(const std::string& line);
 };
 
 class CoreExport FakeUser : public User
 {
  public:
-	FakeUser(const std::string &uid, const std::string& srv) : User(uid, srv, USERTYPE_SERVER)
+	FakeUser(const std::string& uid, Server* srv)
+		: User(uid, srv, USERTYPE_SERVER)
 	{
-		nick = srv;
+		nick = srv->GetName();
 	}
 
-	virtual CullResult cull();
-	virtual void SendText(const std::string& line);
-	virtual const std::string& GetFullHost();
-	virtual const std::string& GetFullRealHost();
+	FakeUser(const std::string& uid, const std::string& sname, const std::string& sdesc)
+		: User(uid, new Server(uid, sname, sdesc), USERTYPE_SERVER)
+	{
+		nick = sname;
+	}
+
+	CullResult cull() CXX11_OVERRIDE;
+	const std::string& GetFullHost() CXX11_OVERRIDE;
+	const std::string& GetFullRealHost() CXX11_OVERRIDE;
 };
 
 /* Faster than dynamic_cast */
 /** Is a local user */
 inline LocalUser* IS_LOCAL(User* u)
 {
-	return u->usertype == USERTYPE_LOCAL ? static_cast<LocalUser*>(u) : NULL;
+	return (u != NULL && u->usertype == USERTYPE_LOCAL) ? static_cast<LocalUser*>(u) : NULL;
 }
 /** Is a remote user */
 inline RemoteUser* IS_REMOTE(User* u)
 {
-	return u->usertype == USERTYPE_REMOTE ? static_cast<RemoteUser*>(u) : NULL;
+	return (u != NULL && u->usertype == USERTYPE_REMOTE) ? static_cast<RemoteUser*>(u) : NULL;
 }
 /** Is a server fakeuser */
 inline FakeUser* IS_SERVER(User* u)
 {
-	return u->usertype == USERTYPE_SERVER ? static_cast<FakeUser*>(u) : NULL;
+	return (u != NULL && u->usertype == USERTYPE_SERVER) ? static_cast<FakeUser*>(u) : NULL;
 }
-/** Is an oper */
-#define IS_OPER(x) (x->oper)
-/** Is away */
-#define IS_AWAY(x) (!x->awaymsg.empty())
 
-/** Derived from Resolver, and performs user forward/reverse lookups.
- */
-class CoreExport UserResolver : public Resolver
+inline bool User::IsModeSet(const ModeHandler* mh) const
 {
- private:
-	/** UUID we are looking up */
-	std::string uuid;
-	/** True if the lookup is forward, false if is a reverse lookup
-	 */
-	bool fwd;
- public:
-	/** Create a resolver.
-	 * @param user The user to begin lookup on
-	 * @param to_resolve The IP or host to resolve
-	 * @param qt The query type
-	 * @param cache Modified by the constructor if the result was cached
-	 */
-	UserResolver(LocalUser* user, std::string to_resolve, QueryType qt, bool &cache);
+	return ((mh->GetId() != ModeParser::MODEID_MAX) && (modes[mh->GetId()]));
+}
 
-	/** Called on successful lookup
-	 * @param result Result string
-	 * @param ttl Time to live for result
-	 * @param cached True if the result was found in the cache
-	 */
-	void OnLookupComplete(const std::string &result, unsigned int ttl, bool cached);
+inline bool User::IsModeSet(UserModeReference& moderef) const
+{
+	if (!moderef)
+		return false;
+	return IsModeSet(*moderef);
+}
 
-	/** Called on failed lookup
-	 * @param e Error code
-	 * @param errormessage Error message string
-	 */
-	void OnError(ResolverError e, const std::string &errormessage);
-};
-
-#endif
+inline void User::SetMode(ModeHandler* mh, bool value)
+{
+	if (mh && mh->GetId() != ModeParser::MODEID_MAX)
+		modes[mh->GetId()] = value;
+}
