@@ -26,23 +26,14 @@
  */
 
 
-/* $Core */
 #include "inspircd.h"
-#include "inspircd_version.h"
 #include <signal.h>
 
 #ifndef _WIN32
-	#include <dirent.h>
 	#include <unistd.h>
 	#include <sys/resource.h>
 	#include <dlfcn.h>
 	#include <getopt.h>
-
-	/* Some systems don't define RUSAGE_SELF. This should fix them. */
-	#ifndef RUSAGE_SELF
-		#define RUSAGE_SELF 0
-	#endif
-
 	#include <pwd.h> // setuid
 	#include <grp.h> // setgid
 #else
@@ -54,13 +45,7 @@
 #include <fstream>
 #include <iostream>
 #include "xline.h"
-#include "bancache.h"
-#include "socketengine.h"
-#include "socket.h"
-#include "command_parse.h"
 #include "exitcodes.h"
-#include "caller.h"
-#include "testsuite.h"
 
 InspIRCd* ServerInstance = NULL;
 
@@ -78,26 +63,17 @@ unsigned const char *national_case_insensitive_map = rfc_case_insensitive_map;
  */
 const char* ExitCodes[] =
 {
-		"No error", /* 0 */
-		"DIE command", /* 1 */
-		"execv() failed", /* 2 */
-		"Internal error", /* 3 */
-		"Config file error", /* 4 */
-		"Logfile error", /* 5 */
-		"POSIX fork failed", /* 6 */
-		"Bad commandline parameters", /* 7 */
-		"No ports could be bound", /* 8 */
-		"Can't write PID file", /* 9 */
-		"SocketEngine could not initialize", /* 10 */
-		"Refusing to start up as root", /* 11 */
-		"Found a <die> tag!", /* 12 */
-		"Couldn't load module on startup", /* 13 */
-		"Could not create windows forked process", /* 14 */
-		"Received SIGTERM", /* 15 */
-		"Bad command handler loaded", /* 16 */
-		"RegisterServiceCtrlHandler failed", /* 17 */
-		"UpdateSCMStatus failed", /* 18 */
-		"CreateEvent failed" /* 19 */
+		"No error",								// 0
+		"DIE command",							// 1
+		"Config file error",					// 2
+		"Logfile error",						// 3
+		"POSIX fork failed",					// 4
+		"Bad commandline parameters",			// 5
+		"Can't write PID file",					// 6
+		"SocketEngine could not initialize",	// 7
+		"Refusing to start up as root",			// 8
+		"Couldn't load module on startup",		// 9
+		"Received SIGTERM"						// 10
 };
 
 template<typename T> static void DeleteZero(T*&n)
@@ -109,21 +85,22 @@ template<typename T> static void DeleteZero(T*&n)
 
 void InspIRCd::Cleanup()
 {
+	// Close all listening sockets
 	for (unsigned int i = 0; i < ports.size(); i++)
 	{
-		/* This calls the constructor and closes the listening socket */
 		ports[i]->cull();
 		delete ports[i];
 	}
 	ports.clear();
 
-	/* Close all client sockets, or the new process inherits them */
-	LocalUserList::reverse_iterator i = Users->local_users.rbegin();
-	while (i != this->Users->local_users.rend())
-	{
-		User* u = *i++;
-		Users->QuitUser(u, "Server shutdown");
-	}
+	// Tell modules that we're shutting down.
+	const std::string quitmsg = "Server shutting down";
+	FOREACH_MOD(OnShutdown, (quitmsg));
+
+	// Disconnect all local users
+	const UserManager::LocalList& list = Users.GetLocalUsers();
+	while (!list.empty())
+		ServerInstance->Users.QuitUser(list.front(), quitmsg);
 
 	GlobalCulls.Apply();
 	Modules->UnloadAll();
@@ -131,127 +108,35 @@ void InspIRCd::Cleanup()
 	/* Delete objects dynamically allocated in constructor (destructor would be more appropriate, but we're likely exiting) */
 	/* Must be deleted before modes as it decrements modelines */
 	if (FakeClient)
+	{
+		delete FakeClient->server;
 		FakeClient->cull();
-	if (Res)
-		Res->cull();
+	}
 	DeleteZero(this->FakeClient);
-	DeleteZero(this->Users);
-	DeleteZero(this->Modes);
 	DeleteZero(this->XLines);
-	DeleteZero(this->Parser);
-	DeleteZero(this->stats);
-	DeleteZero(this->Modules);
-	DeleteZero(this->BanCache);
-	DeleteZero(this->SNO);
 	DeleteZero(this->Config);
-	DeleteZero(this->Res);
-	DeleteZero(this->chanlist);
-	DeleteZero(this->PI);
-	DeleteZero(this->Threads);
-	DeleteZero(this->Timers);
-	DeleteZero(this->SE);
-	/* Close logging */
-	this->Logs->CloseLogs();
-	DeleteZero(this->Logs);
-}
-
-void InspIRCd::Restart(const std::string &reason)
-{
-	/* SendError flushes each client's queue,
-	 * regardless of writeability state
-	 */
-	this->SendError(reason);
-
-	/* Figure out our filename (if theyve renamed it, we're boned) */
-	std::string me;
-
-	char** argv = Config->cmdline.argv;
-
-#ifdef _WIN32
-	char module[MAX_PATH];
-	if (GetModuleFileNameA(NULL, module, MAX_PATH))
-		me = module;
-#else
-	me = argv[0];
-#endif
-
-	this->Cleanup();
-
-	if (execv(me.c_str(), argv) == -1)
-	{
-		/* Will raise a SIGABRT if not trapped */
-		throw CoreException(std::string("Failed to execv()! error: ") + strerror(errno));
-	}
-}
-
-void InspIRCd::ResetMaxBans()
-{
-	for (chan_hash::const_iterator i = chanlist->begin(); i != chanlist->end(); i++)
-		i->second->ResetMaxBans();
-}
-
-/** Because hash_map doesn't free its buckets when we delete items, we occasionally
- * recreate the hash to free them up.
- * We do this by copying the entries from the old hash to a new hash, causing all
- * empty buckets to be weeded out of the hash.
- * Since this is quite expensive, it's not done very often.
- */
-void InspIRCd::RehashUsersAndChans()
-{
-	user_hash* old_users = Users->clientlist;
-	Users->clientlist = new user_hash;
-	for (user_hash::const_iterator n = old_users->begin(); n != old_users->end(); n++)
-		Users->clientlist->insert(*n);
-	delete old_users;
-
-	user_hash* old_uuid = Users->uuidlist;
-	Users->uuidlist = new user_hash;
-	for (user_hash::const_iterator n = old_uuid->begin(); n != old_uuid->end(); n++)
-		Users->uuidlist->insert(*n);
-	delete old_uuid;
-
-	chan_hash* old_chans = chanlist;
-	chanlist = new chan_hash;
-	for (chan_hash::const_iterator n = old_chans->begin(); n != old_chans->end(); n++)
-		chanlist->insert(*n);
-	delete old_chans;
-
-	// Reset the already_sent IDs so we don't wrap it around and drop a message
-	LocalUser::already_sent_id = 0;
-	for (LocalUserList::const_iterator i = Users->local_users.begin(); i != Users->local_users.end(); i++)
-	{
-		(**i).already_sent = 0;
-		(**i).RemoveExpiredInvites();
-	}
-
-	// HACK: ELines are not expired properly at the moment but it can't be fixed as
-	// the 2.0 XLine system is a spaghetti nightmare. Instead we skip over expired
-	// ELines in XLineManager::CheckELines() and expire them here instead.
-	ServerInstance->XLines->GetAll("E");
+	SocketEngine::Deinit();
+	Logs->CloseLogs();
 }
 
 void InspIRCd::SetSignals()
 {
 #ifndef _WIN32
 	signal(SIGALRM, SIG_IGN);
+	signal(SIGCHLD, SIG_IGN);
 	signal(SIGHUP, InspIRCd::SetSignal);
 	signal(SIGPIPE, SIG_IGN);
-	signal(SIGCHLD, SIG_IGN);
-	/* We want E2BIG not a signal! */
+	signal(SIGUSR1, SIG_IGN);
+	signal(SIGUSR2, SIG_IGN);
 	signal(SIGXFSZ, SIG_IGN);
 #endif
 	signal(SIGTERM, InspIRCd::SetSignal);
 }
 
-void InspIRCd::QuickExit(int status)
-{
-	exit(status);
-}
-
 // Required for returning the proper value of EXIT_SUCCESS for the parent process
 static void VoidSignalHandler(int signalreceived)
 {
-	exit(0);
+	exit(EXIT_STATUS_NOERROR);
 }
 
 bool InspIRCd::DaemonSeed()
@@ -260,11 +145,11 @@ bool InspIRCd::DaemonSeed()
 	std::cout << "InspIRCd Process ID: " << con_green << GetCurrentProcessId() << con_reset << std::endl;
 	return true;
 #else
-	// Do not use QuickExit here: It will exit with status SIGTERM which would break e.g. daemon scripts
+	// Do not use exit() here: It will exit with status SIGTERM which would break e.g. daemon scripts
 	signal(SIGTERM, VoidSignalHandler);
 
-	int childpid;
-	if ((childpid = fork ()) < 0)
+	int childpid = fork();
+	if (childpid < 0)
 		return false;
 	else if (childpid > 0)
 	{
@@ -277,7 +162,7 @@ bool InspIRCd::DaemonSeed()
 		 */
 		while (kill(childpid, 0) != -1)
 			sleep(1);
-		exit(0);
+		exit(EXIT_STATUS_NOERROR);
 	}
 	setsid ();
 	std::cout << "InspIRCd Process ID: " << con_green << getpid() << con_reset << std::endl;
@@ -287,13 +172,13 @@ bool InspIRCd::DaemonSeed()
 	rlimit rl;
 	if (getrlimit(RLIMIT_CORE, &rl) == -1)
 	{
-		this->Logs->Log("STARTUP",DEFAULT,"Failed to getrlimit()!");
+		this->Logs->Log("STARTUP", LOG_DEFAULT, "Failed to getrlimit()!");
 		return false;
 	}
 	rl.rlim_cur = rl.rlim_max;
 
 	if (setrlimit(RLIMIT_CORE, &rl) == -1)
-			this->Logs->Log("STARTUP",DEFAULT,"setrlimit() failed, cannot increase coredump size.");
+			this->Logs->Log("STARTUP", LOG_DEFAULT, "setrlimit() failed, cannot increase coredump size.");
 
 	return true;
 #endif
@@ -302,9 +187,13 @@ bool InspIRCd::DaemonSeed()
 void InspIRCd::WritePID(const std::string& filename, bool exitonfail)
 {
 #ifndef _WIN32
-	std::string fname(filename);
-	if (fname.empty())
-		fname = DATA_PATH "/inspircd.pid";
+	if (!ServerInstance->Config->cmdline.writepid)
+	{
+		this->Logs->Log("STARTUP", LOG_DEFAULT, "--nopid specified on command line; PID file not written.");
+		return;
+	}
+
+	std::string fname = ServerInstance->Config->Paths.PrependData(filename.empty() ? "inspircd.pid" : filename);
 	std::ofstream outfile(fname.c_str());
 	if (outfile.is_open())
 	{
@@ -315,7 +204,7 @@ void InspIRCd::WritePID(const std::string& filename, bool exitonfail)
 	{
 		if (exitonfail)
 			std::cout << "Failed to write PID-file '" << fname << "', exiting." << std::endl;
-		this->Logs->Log("STARTUP",DEFAULT,"Failed to write PID-file '%s'%s",fname.c_str(), (exitonfail ? ", exiting." : ""));
+		this->Logs->Log("STARTUP", LOG_DEFAULT, "Failed to write PID-file '%s'%s", fname.c_str(), (exitonfail ? ", exiting." : ""));
 		if (exitonfail)
 			Exit(EXIT_STATUS_PID);
 	}
@@ -323,82 +212,39 @@ void InspIRCd::WritePID(const std::string& filename, bool exitonfail)
 }
 
 InspIRCd::InspIRCd(int argc, char** argv) :
-	 ConfigFileName(CONFIG_PATH "/inspircd.conf"),
+	 ConfigFileName(INSPIRCD_CONFIG_PATH "/inspircd.conf"),
+	 PI(&DefaultProtocolInterface),
 
 	 /* Functor pointer initialisation.
 	  *
 	  * THIS MUST MATCH THE ORDER OF DECLARATION OF THE FUNCTORS, e.g. the methods
 	  * themselves within the class.
 	  */
-	 NICKForced("NICKForced", NULL),
-	 OperQuit("OperQuit", NULL),
-	 GenRandom(&HandleGenRandom),
-	 IsChannel(&HandleIsChannel),
-	 IsSID(&HandleIsSID),
-	 Rehash(&HandleRehash),
-	 IsNick(&HandleIsNick),
-	 IsIdent(&HandleIsIdent),
-	 FloodQuitUser(&HandleFloodQuitUser),
-	 OnCheckExemption(&HandleOnCheckExemption)
+	 GenRandom(&DefaultGenRandom),
+	 IsChannel(&DefaultIsChannel),
+	 IsNick(&DefaultIsNick),
+	 IsIdent(&DefaultIsIdent)
 {
 	ServerInstance = this;
 
-	Extensions.Register(&NICKForced);
-	Extensions.Register(&OperQuit);
-
 	FailedPortList pl;
+	// Flag variables passed to getopt_long() later
 	int do_version = 0, do_nofork = 0, do_debug = 0,
-	    do_nolog = 0, do_root = 0, do_testsuite = 0;    /* flag variables */
-	int c = 0;
+	    do_nolog = 0, do_nopid = 0, do_root = 0;
 
 	// Initialize so that if we exit before proper initialization they're not deleted
-	this->Logs = 0;
-	this->Threads = 0;
-	this->PI = 0;
-	this->Users = 0;
-	this->chanlist = 0;
 	this->Config = 0;
-	this->SNO = 0;
-	this->BanCache = 0;
-	this->Modules = 0;
-	this->stats = 0;
-	this->Timers = 0;
-	this->Parser = 0;
 	this->XLines = 0;
-	this->Modes = 0;
-	this->Res = 0;
 	this->ConfigThread = NULL;
 	this->FakeClient = NULL;
 
 	UpdateTime();
 	this->startup_time = TIME.tv_sec;
 
-	// This must be created first, so other parts of Insp can use it while starting up
-	this->Logs = new LogManager;
-
-	SE = CreateSocketEngine();
-
-	this->Threads = new ThreadEngine;
-
-	/* Default implementation does nothing */
-	this->PI = new ProtocolInterface;
-
-	this->s_signal = 0;
-
-	// Create base manager classes early, so nothing breaks
-	this->Users = new UserManager;
-
-	this->Users->clientlist = new user_hash();
-	this->Users->uuidlist = new user_hash();
-	this->chanlist = new chan_hash();
+	SocketEngine::Init();
 
 	this->Config = new ServerConfig;
-	this->SNO = new SnomaskManager;
-	this->BanCache = new BanCacheManager;
-	this->Modules = new ModuleManager();
-	this->stats = new serverstats();
-	this->Timers = new TimerManager;
-	this->Parser = new CommandParser;
+	dynamic_reference_base::reset_all();
 	this->XLines = new XLineManager;
 
 	this->Config->cmdline.argv = argv;
@@ -424,31 +270,46 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 	srandom(TIME.tv_nsec ^ TIME.tv_sec);
 #endif
 
+	{
+		ServiceProvider* provs[] =
+		{
+			&rfcevents.numeric, &rfcevents.join, &rfcevents.part, &rfcevents.kick, &rfcevents.quit, &rfcevents.nick,
+			&rfcevents.mode, &rfcevents.topic, &rfcevents.privmsg, &rfcevents.invite, &rfcevents.ping, &rfcevents.pong,
+			&rfcevents.error
+		};
+		Modules.AddServices(provs, sizeof(provs)/sizeof(provs[0]));
+	}
+
 	struct option longopts[] =
 	{
 		{ "nofork",	no_argument,		&do_nofork,	1	},
-		{ "logfile",	required_argument,	NULL,		'f'	},
 		{ "config",	required_argument,	NULL,		'c'	},
 		{ "debug",	no_argument,		&do_debug,	1	},
 		{ "nolog",	no_argument,		&do_nolog,	1	},
+		{ "nopid",	no_argument,		&do_nopid,	1	},
 		{ "runasroot",	no_argument,		&do_root,	1	},
 		{ "version",	no_argument,		&do_version,	1	},
-		{ "testsuite",	no_argument,		&do_testsuite,	1	},
 		{ 0, 0, 0, 0 }
 	};
 
+	int c;
 	int index;
-	while ((c = getopt_long(argc, argv, ":c:f:", longopts, &index)) != -1)
+	while ((c = getopt_long(argc, argv, ":c:", longopts, &index)) != -1)
 	{
 		switch (c)
 		{
-			case 'f':
-				/* Log filename was set */
-				Config->cmdline.startup_log = optarg;
-			break;
 			case 'c':
 				/* Config filename was set */
 				ConfigFileName = optarg;
+#ifdef _WIN32
+				TCHAR configPath[MAX_PATH + 1];
+				if (GetFullPathName(optarg, MAX_PATH, configPath, NULL) > 0)
+					ConfigFileName = configPath;
+#else
+				char configPath[PATH_MAX + 1];
+				if (realpath(optarg, configPath))
+					ConfigFileName = configPath;
+#endif
 			break;
 			case 0:
 				/* getopt_long_only() set an int variable, just keep going */
@@ -458,19 +319,16 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 			default:
 				/* Fall through to handle other weird values too */
 				std::cout << "Unknown parameter '" << argv[optind-1] << "'" << std::endl;
-				std::cout << "Usage: " << argv[0] << " [--nofork] [--nolog] [--debug] [--logfile <filename>] " << std::endl <<
-					std::string(static_cast<int>(8+strlen(argv[0])), ' ') << "[--runasroot] [--version] [--config <config>] [--testsuite]" << std::endl;
+				std::cout << "Usage: " << argv[0] << " [--nofork] [--nolog] [--nopid] [--debug] [--config <config>]" << std::endl <<
+					std::string(static_cast<size_t>(8+strlen(argv[0])), ' ') << "[--runasroot] [--version]" << std::endl;
 				Exit(EXIT_STATUS_ARGV);
 			break;
 		}
 	}
 
-	if (do_testsuite)
-		do_nofork = do_debug = true;
-
 	if (do_version)
 	{
-		std::cout << std::endl << VERSION << " r" << REVISION << std::endl;
+		std::cout << std::endl << INSPIRCD_VERSION << std::endl;
 		Exit(EXIT_STATUS_NOERROR);
 	}
 
@@ -484,28 +342,23 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 	Config->cmdline.nofork = (do_nofork != 0);
 	Config->cmdline.forcedebug = (do_debug != 0);
 	Config->cmdline.writelog = !do_nolog;
-	Config->cmdline.TestSuite = (do_testsuite != 0);
+	Config->cmdline.writepid = !do_nopid;
 
 	if (do_debug)
 	{
-		FileWriter* fw = new FileWriter(stdout);
-		FileLogStream* fls = new FileLogStream(RAWIO, fw);
+		FileWriter* fw = new FileWriter(stdout, 1);
+		FileLogStream* fls = new FileLogStream(LOG_RAWIO, fw);
 		Logs->AddLogTypes("*", fls, true);
 	}
-	else if (!this->OpenLog(argv, argc))
-	{
-		std::cout << "ERROR: Could not open initial logfile " << Config->cmdline.startup_log << ": " << strerror(errno) << std::endl << std::endl;
-		Exit(EXIT_STATUS_LOG);
-	}
 
-	if (!ServerConfig::FileExists(ConfigFileName.c_str()))
+	if (!FileSystem::FileExists(ConfigFileName))
 	{
 #ifdef _WIN32
 		/* Windows can (and defaults to) hide file extensions, so let's play a bit nice for windows users. */
 		std::string txtconf = this->ConfigFileName;
 		txtconf.append(".txt");
 
-		if (ServerConfig::FileExists(txtconf.c_str()))
+		if (FileSystem::FileExists(txtconf))
 		{
 			ConfigFileName = txtconf;
 		}
@@ -513,20 +366,13 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 #endif
 		{
 			std::cout << "ERROR: Cannot open config file: " << ConfigFileName << std::endl << "Exiting..." << std::endl;
-			this->Logs->Log("STARTUP",DEFAULT,"Unable to open config file %s", ConfigFileName.c_str());
+			this->Logs->Log("STARTUP", LOG_DEFAULT, "Unable to open config file %s", ConfigFileName.c_str());
 			Exit(EXIT_STATUS_CONFIG);
 		}
 	}
 
-	std::cout << con_green << "Inspire Internet Relay Chat Server" << con_reset << ", compiled on " __DATE__ " at " __TIME__ << std::endl;
-	std::cout << con_green << "(C) InspIRCd Development Team." << con_reset << std::endl << std::endl;
-	std::cout << "Developers:" << std::endl;
-	std::cout << con_green << "\tBrain, FrostyCoolSlug, w00t, Om, Special, peavey" << std::endl;
-	std::cout << "\taquanight, psychon, dz, danieldg, jackmcbarn" << std::endl;
-	std::cout << "\tAttila" << con_reset << std::endl << std::endl;
-	std::cout << "Others:\t\t\t" << con_green << "See /INFO Output" << con_reset << std::endl;
-
-	this->Modes = new ModeParser;
+	std::cout << con_green << "InspIRCd - Internet Relay Chat Daemon" << con_reset << std::endl;
+	std::cout << "For contributors & authors: " << con_green << "See /INFO Output" << con_reset << std::endl;
 
 #ifndef _WIN32
 	if (!do_root)
@@ -552,47 +398,31 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 		if (!this->DaemonSeed())
 		{
 			std::cout << "ERROR: could not go into daemon mode. Shutting down." << std::endl;
-			Logs->Log("STARTUP", DEFAULT, "ERROR: could not go into daemon mode. Shutting down.");
+			Logs->Log("STARTUP", LOG_DEFAULT, "ERROR: could not go into daemon mode. Shutting down.");
 			Exit(EXIT_STATUS_FORK);
 		}
 	}
 
-	SE->RecoverFromFork();
+	SocketEngine::RecoverFromFork();
 
-	/* During startup we don't actually initialize this
-	 * in the thread engine.
+	/* During startup we read the configuration now, not in
+	 * a seperate thread
 	 */
 	this->Config->Read();
 	this->Config->Apply(NULL, "");
 	Logs->OpenFileLogs();
 
-	this->Res = new DNS();
-
-	/*
-	 * Initialise SID/UID.
- 	 * For an explanation as to exactly how this works, and why it works this way, see GetUID().
-	 *   -- w00t
- 	 */
+	// If we don't have a SID, generate one based on the server name and the server description
 	if (Config->sid.empty())
-	{
-		// Generate one
-		unsigned int sid = 0;
-		char sidstr[4];
+		Config->sid = UIDGenerator::GenerateSID(Config->ServerName, Config->ServerDesc);
 
-		for (const char* x = Config->ServerName.c_str(); *x; ++x)
-			sid = 5 * sid + *x;
-		for (const char* y = Config->ServerDesc.c_str(); *y; ++y)
-			sid = 5 * sid + *y;
-		sprintf(sidstr, "%03d", sid % 1000);
+	// Initialize the UID generator with our sid
+	this->UIDGen.init(Config->sid);
 
-		Config->sid = sidstr;
-	}
+	// Create the server user for this server
+	this->FakeClient = new FakeUser(Config->sid, Config->ServerName, Config->ServerDesc);
 
-	/* set up fake client again this time with the correct uid */
-	this->FakeClient = new FakeUser(Config->sid, Config->ServerName);
-
-	// Get XLine to do it's thing.
-	this->XLines->CheckELines();
+	// This is needed as all new XLines are marked pending until ApplyLines() is called
 	this->XLines->ApplyLines();
 
 	int bounditems = BindPorts(pl);
@@ -601,9 +431,8 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 
 	this->Modules->LoadAll();
 
-	/* Just in case no modules were loaded - fix for bug #101 */
-	this->BuildISupport();
-	Config->ApplyDisabledCommands(Config->DisabledCommands);
+	// Build ISupport as ModuleManager::LoadAll() does not do it
+	this->ISupport.Build();
 
 	if (!pl.empty())
 	{
@@ -613,13 +442,13 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 		int j = 1;
 		for (FailedPortList::iterator i = pl.begin(); i != pl.end(); i++, j++)
 		{
-			std::cout << j << ".\tAddress: " << (i->first.empty() ? "<all>" : i->first) << " \tReason: " << i->second << std::endl;
+			std::cout << j << ".\tAddress: " << i->first.str() << " \tReason: " << strerror(i->second) << std::endl;
 		}
 
 		std::cout << std::endl << "Hint: Try using a public IP instead of blank or *" << std::endl;
 	}
 
-	std::cout << "InspIRCd is now running as '" << Config->ServerName << "'[" << Config->GetSID() << "] with " << SE->GetMaxFds() << " max open sockets" << std::endl;
+	std::cout << "InspIRCd is now running as '" << Config->ServerName << "'[" << Config->GetSID() << "] with " << SocketEngine::GetMaxFds() << " max open sockets" << std::endl;
 
 #ifndef _WIN32
 	if (!Config->cmdline.nofork)
@@ -627,7 +456,7 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 		if (kill(getppid(), SIGTERM) == -1)
 		{
 			std::cout << "Error killing parent process: " << strerror(errno) << std::endl;
-			Logs->Log("STARTUP", DEFAULT, "Error killing parent process: %s",strerror(errno));
+			Logs->Log("STARTUP", LOG_DEFAULT, "Error killing parent process: %s",strerror(errno));
 		}
 	}
 
@@ -641,7 +470,7 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 	 *
 	 *    -- nenolod
 	 */
-	if ((!do_nofork) && (!do_testsuite) && (!Config->cmdline.forcedebug))
+	if ((!do_nofork) && (!Config->cmdline.forcedebug))
 	{
 		int fd = open("/dev/null", O_RDWR);
 
@@ -650,16 +479,16 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 		fclose(stdout);
 
 		if (dup2(fd, STDIN_FILENO) < 0)
-			Logs->Log("STARTUP", DEFAULT, "Failed to dup /dev/null to stdin.");
+			Logs->Log("STARTUP", LOG_DEFAULT, "Failed to dup /dev/null to stdin.");
 		if (dup2(fd, STDOUT_FILENO) < 0)
-			Logs->Log("STARTUP", DEFAULT, "Failed to dup /dev/null to stdout.");
+			Logs->Log("STARTUP", LOG_DEFAULT, "Failed to dup /dev/null to stdout.");
 		if (dup2(fd, STDERR_FILENO) < 0)
-			Logs->Log("STARTUP", DEFAULT, "Failed to dup /dev/null to stderr.");
+			Logs->Log("STARTUP", LOG_DEFAULT, "Failed to dup /dev/null to stderr.");
 		close(fd);
 	}
 	else
 	{
-		Logs->Log("STARTUP", DEFAULT,"Keeping pseudo-tty open as we are running in the foreground.");
+		Logs->Log("STARTUP", LOG_DEFAULT, "Keeping pseudo-tty open as we are running in the foreground.");
 	}
 #else
 	/* Set win32 service as running, if we are running as a service */
@@ -671,68 +500,53 @@ InspIRCd::InspIRCd(int argc, char** argv) :
 		FreeConsole();
 	}
 
-	QueryPerformanceFrequency(&stats->QPFrequency);
+	QueryPerformanceFrequency(&stats.QPFrequency);
 #endif
 
-	Logs->Log("STARTUP", DEFAULT, "Startup complete as '%s'[%s], %d max open sockets", Config->ServerName.c_str(),Config->GetSID().c_str(), SE->GetMaxFds());
+	Logs->Log("STARTUP", LOG_DEFAULT, "Startup complete as '%s'[%s], %lu max open sockets", Config->ServerName.c_str(),Config->GetSID().c_str(), SocketEngine::GetMaxFds());
 
 #ifndef _WIN32
-	std::string SetUser = Config->ConfValue("security")->getString("runasuser");
-	std::string SetGroup = Config->ConfValue("security")->getString("runasgroup");
+	ConfigTag* security = Config->ConfValue("security");
+
+	const std::string SetGroup = security->getString("runasgroup");
 	if (!SetGroup.empty())
 	{
-		int ret;
-
-		// setgroups
-		ret = setgroups(0, NULL);
-
-		if (ret == -1)
+		errno = 0;
+		if (setgroups(0, NULL) == -1)
 		{
-			this->Logs->Log("SETGROUPS", DEFAULT, "setgroups() failed (wtf?): %s", strerror(errno));
-			this->QuickExit(0);
+			this->Logs->Log("STARTUP", LOG_DEFAULT, "setgroups() failed (wtf?): %s", strerror(errno));
+			exit(EXIT_STATUS_CONFIG);
 		}
 
-		// setgid
-		struct group *g;
-
-		errno = 0;
-		g = getgrnam(SetGroup.c_str());
-
+		struct group* g = getgrnam(SetGroup.c_str());
 		if (!g)
 		{
-			this->Logs->Log("SETGUID", DEFAULT, "getgrnam(%s) failed (wrong group?): %s", SetGroup.c_str(), strerror(errno));
-			this->QuickExit(0);
+			this->Logs->Log("STARTUP", LOG_DEFAULT, "getgrnam(%s) failed (wrong group?): %s", SetGroup.c_str(), strerror(errno));
+			exit(EXIT_STATUS_CONFIG);
 		}
 
-		ret = setgid(g->gr_gid);
-
-		if (ret == -1)
+		if (setgid(g->gr_gid) == -1)
 		{
-			this->Logs->Log("SETGUID", DEFAULT, "setgid() failed (wrong group?): %s", strerror(errno));
-			this->QuickExit(0);
+			this->Logs->Log("STARTUP", LOG_DEFAULT, "setgid(%d) failed (wrong group?): %s", g->gr_gid, strerror(errno));
+			exit(EXIT_STATUS_CONFIG);
 		}
 	}
 
+	const std::string SetUser = security->getString("runasuser");
 	if (!SetUser.empty())
 	{
-		// setuid
-		struct passwd *u;
-
 		errno = 0;
-		u = getpwnam(SetUser.c_str());
-
+		struct passwd* u = getpwnam(SetUser.c_str());
 		if (!u)
 		{
-			this->Logs->Log("SETGUID", DEFAULT, "getpwnam(%s) failed (wrong user?): %s", SetUser.c_str(), strerror(errno));
-			this->QuickExit(0);
+			this->Logs->Log("STARTUP", LOG_DEFAULT, "getpwnam(%s) failed (wrong user?): %s", SetUser.c_str(), strerror(errno));
+			exit(EXIT_STATUS_CONFIG);
 		}
 
-		int ret = setuid(u->pw_uid);
-
-		if (ret == -1)
+		if (setuid(u->pw_uid) == -1)
 		{
-			this->Logs->Log("SETGUID", DEFAULT, "setuid() failed (wrong user?): %s", strerror(errno));
-			this->QuickExit(0);
+			this->Logs->Log("STARTUP", LOG_DEFAULT, "setuid(%d) failed (wrong user?): %s", u->pw_uid, strerror(errno));
+			exit(EXIT_STATUS_CONFIG);
 		}
 	}
 
@@ -760,16 +574,8 @@ void InspIRCd::UpdateTime()
 #endif
 }
 
-int InspIRCd::Run()
+void InspIRCd::Run()
 {
-	/* See if we're supposed to be running the test suite rather than entering the mainloop */
-	if (Config->cmdline.TestSuite)
-	{
-		TestSuite* ts = new TestSuite;
-		delete ts;
-		Exit(0);
-	}
-
 	UpdateTime();
 	time_t OLDTIME = TIME.tv_sec;
 
@@ -783,7 +589,7 @@ int InspIRCd::Run()
 		if (this->ConfigThread && this->ConfigThread->IsDone())
 		{
 			/* Rehash has completed */
-			this->Logs->Log("CONFIG",DEBUG,"Detected ConfigThread exiting, tidying up...");
+			this->Logs->Log("CONFIG", LOG_DEBUG, "Detected ConfigThread exiting, tidying up...");
 
 			this->ConfigThread->Finish();
 
@@ -803,45 +609,50 @@ int InspIRCd::Run()
 		{
 #ifndef _WIN32
 			getrusage(RUSAGE_SELF, &ru);
-			stats->LastSampled = TIME;
-			stats->LastCPU = ru.ru_utime;
+			stats.LastSampled = TIME;
+			stats.LastCPU = ru.ru_utime;
 #else
-			if(QueryPerformanceCounter(&stats->LastSampled))
+			if(QueryPerformanceCounter(&stats.LastSampled))
 			{
 				FILETIME CreationTime;
 				FILETIME ExitTime;
 				FILETIME KernelTime;
 				FILETIME UserTime;
 				GetProcessTimes(GetCurrentProcess(), &CreationTime, &ExitTime, &KernelTime, &UserTime);
-				stats->LastCPU.dwHighDateTime = KernelTime.dwHighDateTime + UserTime.dwHighDateTime;
-				stats->LastCPU.dwLowDateTime = KernelTime.dwLowDateTime + UserTime.dwLowDateTime;
+				stats.LastCPU.dwHighDateTime = KernelTime.dwHighDateTime + UserTime.dwHighDateTime;
+				stats.LastCPU.dwLowDateTime = KernelTime.dwLowDateTime + UserTime.dwLowDateTime;
 			}
 #endif
 
-			/* Allow a buffer of two seconds drift on this so that ntpdate etc dont harass admins */
-			if (TIME.tv_sec < OLDTIME - 2)
+			if (Config->TimeSkipWarn)
 			{
-				SNO->WriteToSnoMask('d', "\002EH?!\002 -- Time is flowing BACKWARDS in this dimension! Clock drifted backwards %lu secs.", (unsigned long)(OLDTIME-TIME.tv_sec));
-			}
-			else if (TIME.tv_sec > OLDTIME + 2)
-			{
-				SNO->WriteToSnoMask('d', "\002EH?!\002 -- Time is jumping FORWARDS! Clock skipped %lu secs.", (unsigned long)(TIME.tv_sec - OLDTIME));
+				time_t timediff = TIME.tv_sec - OLDTIME;
+
+				if (timediff > Config->TimeSkipWarn)
+					SNO->WriteToSnoMask('a', "\002Performance warning!\002 Server clock jumped forwards by %lu seconds!", timediff);
+
+				else if (timediff < -Config->TimeSkipWarn)
+					SNO->WriteToSnoMask('a', "\002Performance warning!\002 Server clock jumped backwards by %lu seconds!", labs(timediff));
 			}
 
 			OLDTIME = TIME.tv_sec;
 
 			if ((TIME.tv_sec % 3600) == 0)
 			{
-				this->RehashUsersAndChans();
-				FOREACH_MOD(I_OnGarbageCollect, OnGarbageCollect());
+				FOREACH_MOD(OnGarbageCollect, ());
+
+				// HACK: ELines are not expired properly at the moment but it can't be fixed as
+				// the 2.0 XLine system is a spaghetti nightmare. Instead we skip over expired
+				// ELines in XLineManager::CheckELines() and expire them here instead.
+				XLines->GetAll("E");
 			}
 
-			Timers->TickTimers(TIME.tv_sec);
-			this->DoBackgroundUserStuff();
+			Timers.TickTimers(TIME.tv_sec);
+			Users->DoBackgroundUserStuff();
 
 			if ((TIME.tv_sec % 5) == 0)
 			{
-				FOREACH_MOD(I_OnBackgroundTimer,OnBackgroundTimer(TIME.tv_sec));
+				FOREACH_MOD(OnBackgroundTimer, (TIME.tv_sec));
 				SNO->FlushSnotices();
 			}
 		}
@@ -853,8 +664,8 @@ int InspIRCd::Run()
 		 * This will cause any read or write events to be
 		 * dispatched to their handlers.
 		 */
-		this->SE->DispatchTrialWrites();
-		this->SE->DispatchEvents();
+		SocketEngine::DispatchTrialWrites();
+		SocketEngine::DispatchEvents();
 
 		/* if any users were quit, take them out */
 		GlobalCulls.Apply();
@@ -866,25 +677,6 @@ int InspIRCd::Run()
 			s_signal = 0;
 		}
 	}
-
-	return 0;
-}
-
-/**********************************************************************************/
-
-/**
- * An ircd in five lines! bwahahaha. ahahahahaha. ahahah *cough*.
- */
-
-/* this returns true when all modules are satisfied that the user should be allowed onto the irc server
- * (until this returns true, a user will block in the waiting state, waiting to connect up to the
- * registration timeout maximum seconds)
- */
-bool InspIRCd::AllModulesReportReady(LocalUser* user)
-{
-	ModResult res;
-	FIRST_MOD_RESULT(OnCheckReady, res, (user));
-	return (res == MOD_RES_PASSTHRU);
 }
 
 sig_atomic_t InspIRCd::s_signal = 0;
