@@ -19,13 +19,34 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/// $CompilerFlags: execute("mysql_config --include" "MYSQL_CXXFLAGS")
+/// $LinkerFlags: execute("mysql_config --libs_r" "MYSQL_LDFLAGS" "-lmysqlclient")
 
-/* Stop mysql wanting to use long long */
-#define NO_CLIENT_LONG_LONG
+/// $PackageInfo: require_system("arch") mariadb-libs
+/// $PackageInfo: require_system("centos" "6.0" "6.99") mysql-devel
+/// $PackageInfo: require_system("centos" "7.0") mariadb-devel
+/// $PackageInfo: require_system("darwin") mysql-connector-c
+/// $PackageInfo: require_system("debian") libmysqlclient-dev
+/// $PackageInfo: require_system("ubuntu") libmysqlclient-dev
+
+#ifdef __GNUC__
+# pragma GCC diagnostic push
+#endif
+
+// Fix warnings about the use of `long long` on C++03.
+#if defined __clang__
+# pragma clang diagnostic ignored "-Wc++11-long-long"
+#elif defined __GNUC__
+# pragma GCC diagnostic ignored "-Wlong-long"
+#endif
 
 #include "inspircd.h"
 #include <mysql.h>
-#include "sql.h"
+#include "modules/sql.h"
+
+#ifdef __GNUC__
+# pragma GCC diagnostic pop
+#endif
 
 #ifdef _WIN32
 # pragma comment(lib, "libmysql.lib")
@@ -33,10 +54,6 @@
 
 /* VERSION 3 API: With nonblocking (threaded) requests */
 
-/* $ModDesc: SQL Service Provider module for all other m_sql* modules */
-/* $CompileFlags: exec("mysql_config --include") */
-/* $LinkerFlags: exec("mysql_config --libs_r") rpath("mysql_config --libs_r") */
-
 /* THE NONBLOCKING MYSQL API!
  *
  * MySQL provides no nonblocking (asyncronous) API of its own, and its developers recommend
@@ -59,7 +76,7 @@
  * The ircd thread then mutexes the queue once more, reads the outbound response off the head
  * of the queue, and sends it on its way to the original calling module.
  *
- * XXX: You might be asking "why doesnt he just send the response from within the worker thread?"
+ * XXX: You might be asking "why doesnt it just send the response from within the worker thread?"
  * The answer to this is simple. The majority of InspIRCd, and in fact most ircd's are not
  * threadsafe. This module is designed to be threadsafe and is careful with its use of threads,
  * however, if we were to call a module's OnRequest even from within a thread which was not the
@@ -75,20 +92,20 @@ class DispatcherThread;
 
 struct QQueueItem
 {
-	SQLQuery* q;
+	SQL::Query* q;
 	std::string query;
 	SQLConnection* c;
-	QQueueItem(SQLQuery* Q, const std::string& S, SQLConnection* C) : q(Q), query(S), c(C) {}
+	QQueueItem(SQL::Query* Q, const std::string& S, SQLConnection* C) : q(Q), query(S), c(C) {}
 };
 
 struct RQueueItem
 {
-	SQLQuery* q;
+	SQL::Query* q;
 	MySQLresult* r;
-	RQueueItem(SQLQuery* Q, MySQLresult* R) : q(Q), r(R) {}
+	RQueueItem(SQL::Query* Q, MySQLresult* R) : q(Q), r(R) {}
 };
 
-typedef std::map<std::string, SQLConnection*> ConnMap;
+typedef insp::flat_map<std::string, SQLConnection*> ConnMap;
 typedef std::deque<QQueueItem> QueryQueue;
 typedef std::deque<RQueueItem> ResultQueue;
 
@@ -103,11 +120,11 @@ class ModuleSQL : public Module
 	ConnMap connections; // main thread only
 
 	ModuleSQL();
-	void init();
+	void init() CXX11_OVERRIDE;
 	~ModuleSQL();
-	void OnRehash(User* user);
-	void OnUnloadModule(Module* mod);
-	Version GetVersion();
+	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE;
+	void OnUnloadModule(Module* mod) CXX11_OVERRIDE;
+	Version GetVersion() CXX11_OVERRIDE;
 };
 
 class DispatcherThread : public SocketThread
@@ -117,8 +134,8 @@ class DispatcherThread : public SocketThread
  public:
 	DispatcherThread(ModuleSQL* CreatorModule) : Parent(CreatorModule) { }
 	~DispatcherThread() { }
-	virtual void Run();
-	virtual void OnNotify();
+	void Run() CXX11_OVERRIDE;
+	void OnNotify() CXX11_OVERRIDE;
 };
 
 #if !defined(MYSQL_VERSION_ID) || MYSQL_VERSION_ID<32224
@@ -127,16 +144,16 @@ class DispatcherThread : public SocketThread
 
 /** Represents a mysql result set
  */
-class MySQLresult : public SQLResult
+class MySQLresult : public SQL::Result
 {
  public:
-	SQLerror err;
+	SQL::Error err;
 	int currentrow;
 	int rows;
 	std::vector<std::string> colnames;
-	std::vector<SQLEntries> fieldlists;
+	std::vector<SQL::Row> fieldlists;
 
-	MySQLresult(MYSQL_RES* res, int affected_rows) : err(SQL_NO_ERROR), currentrow(0), rows(0)
+	MySQLresult(MYSQL_RES* res, int affected_rows) : err(SQL::SUCCESS), currentrow(0), rows(0)
 	{
 		if (affected_rows >= 1)
 		{
@@ -165,9 +182,9 @@ class MySQLresult : public SQLResult
 					{
 						std::string a = (fields[field_count].name ? fields[field_count].name : "");
 						if (row[field_count])
-							fieldlists[n].push_back(SQLEntry(row[field_count]));
+							fieldlists[n].push_back(SQL::Field(row[field_count]));
 						else
-							fieldlists[n].push_back(SQLEntry());
+							fieldlists[n].push_back(SQL::Field());
 						colnames.push_back(a);
 						field_count++;
 					}
@@ -179,35 +196,44 @@ class MySQLresult : public SQLResult
 		}
 	}
 
-	MySQLresult(SQLerror& e) : err(e)
+	MySQLresult(SQL::Error& e) : err(e)
 	{
 
 	}
 
-	~MySQLresult()
-	{
-	}
-
-	virtual int Rows()
+	int Rows() CXX11_OVERRIDE
 	{
 		return rows;
 	}
 
-	virtual void GetCols(std::vector<std::string>& result)
+	void GetCols(std::vector<std::string>& result) CXX11_OVERRIDE
 	{
 		result.assign(colnames.begin(), colnames.end());
 	}
 
-	virtual SQLEntry GetValue(int row, int column)
+	bool HasColumn(const std::string& column, size_t& index) CXX11_OVERRIDE
+	{
+		for (size_t i = 0; i < colnames.size(); ++i)
+		{
+			if (colnames[i] == column)
+			{
+				index = i;
+				return true;
+			}
+		}
+		return false;
+	}
+
+	SQL::Field GetValue(int row, int column)
 	{
 		if ((row >= 0) && (row < rows) && (column >= 0) && (column < (int)fieldlists[row].size()))
 		{
 			return fieldlists[row][column];
 		}
-		return SQLEntry();
+		return SQL::Field();
 	}
 
-	virtual bool GetRow(SQLEntries& result)
+	bool GetRow(SQL::Row& result) CXX11_OVERRIDE
 	{
 		if (currentrow < rows)
 		{
@@ -225,15 +251,40 @@ class MySQLresult : public SQLResult
 
 /** Represents a connection to a mysql database
  */
-class SQLConnection : public SQLProvider
+class SQLConnection : public SQL::Provider
 {
+ private:
+	bool EscapeString(SQL::Query* query, const std::string& in, std::string& out)
+	{
+		// In the worst case each character may need to be encoded as using two bytes and one
+		// byte is the NUL terminator.
+		std::vector<char> buffer(in.length() * 2 + 1);
+
+		// The return value of mysql_escape_string() is either an error or the length of the
+		// encoded string not including the NUL terminator.
+		//
+		// Unfortunately, someone genius decided that mysql_escape_string should return an
+		// unsigned type even though -1 is returned on error so checking whether an error
+		// happened is a bit cursed.
+		unsigned long escapedsize = mysql_escape_string(&buffer[0], in.c_str(), in.length());
+		if (escapedsize == static_cast<unsigned long>(-1))
+		{
+			SQL::Error err(SQL::QSEND_FAIL, InspIRCd::Format("%u: %s", mysql_errno(connection), mysql_error(connection)));
+			query->OnError(err);
+			return false;
+		}
+
+		out.append(&buffer[0], escapedsize);
+		return true;
+	}
+
  public:
 	reference<ConfigTag> config;
 	MYSQL *connection;
 	Mutex lock;
 
 	// This constructor creates an SQLConnection object with the given credentials, but does not connect yet.
-	SQLConnection(Module* p, ConfigTag* tag) : SQLProvider(p, "SQL/" + tag->getString("id")),
+	SQLConnection(Module* p, ConfigTag* tag) : SQL::Provider(p, "SQL/" + tag->getString("id")),
 		config(tag), connection(NULL)
 	{
 	}
@@ -254,10 +305,16 @@ class SQLConnection : public SQLProvider
 		std::string user = config->getString("user");
 		std::string pass = config->getString("pass");
 		std::string dbname = config->getString("name");
-		int port = config->getInt("port");
+		unsigned int port = config->getUInt("port", 3306);
 		bool rv = mysql_real_connect(connection, host.c_str(), user.c_str(), pass.c_str(), dbname.c_str(), port, NULL, 0);
 		if (!rv)
 			return rv;
+
+		// Enable character set settings
+		std::string charset = config->getString("charset");
+		if ((!charset.empty()) && (mysql_set_character_set(connection, charset.c_str())))
+			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "WARNING: Could not set character set to \"%s\"", charset.c_str());
+
 		std::string initquery;
 		if (config->readString("initialquery", initquery))
 		{
@@ -286,7 +343,7 @@ class SQLConnection : public SQLProvider
 		{
 			/* XXX: See /usr/include/mysql/mysqld_error.h for a list of
 			 * possible error numbers and error messages */
-			SQLerror e(SQL_QREPLY_FAIL, ConvToStr(mysql_errno(connection)) + ": " + mysql_error(connection));
+			SQL::Error e(SQL::QREPLY_FAIL, InspIRCd::Format("%u: %s", mysql_errno(connection), mysql_error(connection)));
 			return new MySQLresult(e);
 		}
 	}
@@ -308,14 +365,15 @@ class SQLConnection : public SQLProvider
 		mysql_close(connection);
 	}
 
-	void submit(SQLQuery* q, const std::string& qs)
+	void Submit(SQL::Query* q, const std::string& qs) CXX11_OVERRIDE
 	{
+		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Executing MySQL query: " + qs);
 		Parent()->Dispatcher->LockQueue();
 		Parent()->qq.push_back(QQueueItem(q, qs, this));
 		Parent()->Dispatcher->UnlockQueueWakeup();
 	}
 
-	void submit(SQLQuery* call, const std::string& q, const ParamL& p)
+	void Submit(SQL::Query* call, const std::string& q, const SQL::ParamList& p) CXX11_OVERRIDE
 	{
 		std::string res;
 		unsigned int param = 0;
@@ -323,27 +381,13 @@ class SQLConnection : public SQLProvider
 		{
 			if (q[i] != '?')
 				res.push_back(q[i]);
-			else
-			{
-				if (param < p.size())
-				{
-					std::string parm = p[param++];
-					// In the worst case, each character may need to be encoded as using two bytes,
-					// and one byte is the terminating null
-					std::vector<char> buffer(parm.length() * 2 + 1);
-
-					// The return value of mysql_escape_string() is the length of the encoded string,
-					// not including the terminating null
-					unsigned long escapedsize = mysql_escape_string(&buffer[0], parm.c_str(), parm.length());
-//					mysql_real_escape_string(connection, queryend, paramscopy[paramnum].c_str(), paramscopy[paramnum].length());
-					res.append(&buffer[0], escapedsize);
-				}
-			}
+			else if (param < p.size() && !EscapeString(call, p[param++], res))
+				return;
 		}
-		submit(call, res);
+		Submit(call, res);
 	}
 
-	void submit(SQLQuery* call, const std::string& q, const ParamM& p)
+	void Submit(SQL::Query* call, const std::string& q, const SQL::ParamMap& p) CXX11_OVERRIDE
 	{
 		std::string res;
 		for(std::string::size_type i = 0; i < q.length(); i++)
@@ -358,18 +402,12 @@ class SQLConnection : public SQLProvider
 					field.push_back(q[i++]);
 				i--;
 
-				ParamM::const_iterator it = p.find(field);
-				if (it != p.end())
-				{
-					std::string parm = it->second;
-					// NOTE: See above
-					std::vector<char> buffer(parm.length() * 2 + 1);
-					unsigned long escapedsize = mysql_escape_string(&buffer[0], parm.c_str(), parm.length());
-					res.append(&buffer[0], escapedsize);
-				}
+				SQL::ParamMap::const_iterator it = p.find(field);
+				if (it != p.end() && !EscapeString(call, it->second, res))
+					return;
 			}
 		}
-		submit(call, res);
+		Submit(call, res);
 	}
 };
 
@@ -380,13 +418,11 @@ ModuleSQL::ModuleSQL()
 
 void ModuleSQL::init()
 {
+	if (mysql_library_init(0, NULL, NULL))
+		throw ModuleException("Unable to initialise the MySQL library!");
+
 	Dispatcher = new DispatcherThread(this);
-	ServerInstance->Threads->Start(Dispatcher);
-
-	Implementation eventlist[] = { I_OnRehash, I_OnUnloadModule };
-	ServerInstance->Modules->Attach(eventlist, this, sizeof(eventlist)/sizeof(Implementation));
-
-	OnRehash(NULL);
+	ServerInstance->Threads.Start(Dispatcher);
 }
 
 ModuleSQL::~ModuleSQL()
@@ -397,19 +433,22 @@ ModuleSQL::~ModuleSQL()
 		Dispatcher->OnNotify();
 		delete Dispatcher;
 	}
+
 	for(ConnMap::iterator i = connections.begin(); i != connections.end(); i++)
 	{
 		delete i->second;
 	}
+
+	mysql_library_end();
 }
 
-void ModuleSQL::OnRehash(User* user)
+void ModuleSQL::ReadConfig(ConfigStatus& status)
 {
 	ConnMap conns;
 	ConfigTagList tags = ServerInstance->Config->ConfTags("database");
 	for(ConfigIter i = tags.first; i != tags.second; i++)
 	{
-		if (i->second->getString("module", "mysql") != "mysql")
+		if (!stdalgo::string::equalsci(i->second->getString("module"), "mysql"))
 			continue;
 		std::string id = i->second->getString("id");
 		ConnMap::iterator curr = connections.find(id);
@@ -428,7 +467,7 @@ void ModuleSQL::OnRehash(User* user)
 
 	// now clean up the deleted databases
 	Dispatcher->LockQueue();
-	SQLerror err(SQL_BAD_DBID);
+	SQL::Error err(SQL::BAD_DBID);
 	for(ConnMap::iterator i = connections.begin(); i != connections.end(); i++)
 	{
 		ServerInstance->Modules->DelService(*i->second);
@@ -455,7 +494,7 @@ void ModuleSQL::OnRehash(User* user)
 
 void ModuleSQL::OnUnloadModule(Module* mod)
 {
-	SQLerror err(SQL_BAD_DBID);
+	SQL::Error err(SQL::BAD_DBID);
 	Dispatcher->LockQueue();
 	unsigned int i = qq.size();
 	while (i > 0)
@@ -482,7 +521,7 @@ void ModuleSQL::OnUnloadModule(Module* mod)
 
 Version ModuleSQL::GetVersion()
 {
-	return Version("MySQL support", VF_VENDOR);
+	return Version("Provides MySQL support", VF_VENDOR);
 }
 
 void DispatcherThread::Run()
@@ -535,7 +574,7 @@ void DispatcherThread::OnNotify()
 	for(ResultQueue::iterator i = Parent->rq.begin(); i != Parent->rq.end(); i++)
 	{
 		MySQLresult* res = i->r;
-		if (res->err.id == SQL_NO_ERROR)
+		if (res->err.code == SQL::SUCCESS)
 			i->q->OnResult(*res);
 		else
 			i->q->OnError(res->err);
