@@ -19,45 +19,60 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/// $CompilerFlags: find_compiler_flags("sqlite3")
+/// $LinkerFlags: find_linker_flags("sqlite3" "-lsqlite3")
+
+/// $PackageInfo: require_system("arch") pkgconf sqlite
+/// $PackageInfo: require_system("centos") pkgconfig sqlite-devel
+/// $PackageInfo: require_system("darwin") pkg-config sqlite3
+/// $PackageInfo: require_system("debian") libsqlite3-dev pkg-config
+/// $PackageInfo: require_system("ubuntu") libsqlite3-dev pkg-config
 
 #include "inspircd.h"
+#include "modules/sql.h"
+
+#ifdef __GNUC__
+# pragma GCC diagnostic push
+#endif
+
+// Fix warnings about the use of `long long` on C++03.
+#if defined __clang__
+# pragma clang diagnostic ignored "-Wc++11-long-long"
+#elif defined __GNUC__
+# pragma GCC diagnostic ignored "-Wlong-long"
+#endif
+
 #include <sqlite3.h>
-#include "sql.h"
+
+#ifdef __GNUC__
+# pragma GCC diagnostic pop
+#endif
 
 #ifdef _WIN32
 # pragma comment(lib, "sqlite3.lib")
 #endif
 
-/* $ModDesc: sqlite3 provider */
-/* $CompileFlags: pkgconfversion("sqlite3","3.3") pkgconfincludes("sqlite3","/sqlite3.h","") */
-/* $LinkerFlags: pkgconflibs("sqlite3","/libsqlite3.so","-lsqlite3") */
-/* $NoPedantic */
-
 class SQLConn;
-typedef std::map<std::string, SQLConn*> ConnMap;
+typedef insp::flat_map<std::string, SQLConn*> ConnMap;
 
-class SQLite3Result : public SQLResult
+class SQLite3Result : public SQL::Result
 {
  public:
 	int currentrow;
 	int rows;
 	std::vector<std::string> columns;
-	std::vector<SQLEntries> fieldlists;
+	std::vector<SQL::Row> fieldlists;
 
 	SQLite3Result() : currentrow(0), rows(0)
 	{
 	}
 
-	~SQLite3Result()
-	{
-	}
-
-	virtual int Rows()
+	int Rows() CXX11_OVERRIDE
 	{
 		return rows;
 	}
 
-	virtual bool GetRow(SQLEntries& result)
+	bool GetRow(SQL::Row& result) CXX11_OVERRIDE
 	{
 		if (currentrow < rows)
 		{
@@ -72,20 +87,32 @@ class SQLite3Result : public SQLResult
 		}
 	}
 
-	virtual void GetCols(std::vector<std::string>& result)
+	void GetCols(std::vector<std::string>& result) CXX11_OVERRIDE
 	{
 		result.assign(columns.begin(), columns.end());
 	}
+
+	bool HasColumn(const std::string& column, size_t& index) CXX11_OVERRIDE
+	{
+		for (size_t i = 0; i < columns.size(); ++i)
+		{
+			if (columns[i] == column)
+			{
+				index = i;
+				return true;
+			}
+		}
+		return false;
+	}
 };
 
-class SQLConn : public SQLProvider
+class SQLConn : public SQL::Provider
 {
- private:
 	sqlite3* conn;
 	reference<ConfigTag> config;
 
  public:
-	SQLConn(Module* Parent, ConfigTag* tag) : SQLProvider(Parent, "SQL/" + tag->getString("id")), config(tag)
+	SQLConn(Module* Parent, ConfigTag* tag) : SQL::Provider(Parent, "SQL/" + tag->getString("id")), config(tag)
 	{
 		std::string host = tag->getString("hostname");
 		if (sqlite3_open_v2(host.c_str(), &conn, SQLITE_OPEN_READWRITE, 0) != SQLITE_OK)
@@ -93,7 +120,7 @@ class SQLConn : public SQLProvider
 			// Even in case of an error conn must be closed
 			sqlite3_close(conn);
 			conn = NULL;
-			ServerInstance->Logs->Log("m_sqlite3",DEFAULT, "WARNING: Could not open DB with id: " + tag->getString("id"));
+			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "WARNING: Could not open DB with id: " + tag->getString("id"));
 		}
 	}
 
@@ -106,14 +133,14 @@ class SQLConn : public SQLProvider
 		}
 	}
 
-	void Query(SQLQuery* query, const std::string& q)
+	void Query(SQL::Query* query, const std::string& q)
 	{
 		SQLite3Result res;
 		sqlite3_stmt *stmt;
 		int err = sqlite3_prepare_v2(conn, q.c_str(), q.length(), &stmt, NULL);
 		if (err != SQLITE_OK)
 		{
-			SQLerror error(SQL_QSEND_FAIL, sqlite3_errmsg(conn));
+			SQL::Error error(SQL::QSEND_FAIL, sqlite3_errmsg(conn));
 			query->OnError(error);
 			return;
 		}
@@ -135,7 +162,7 @@ class SQLConn : public SQLProvider
 				{
 					const char* txt = (const char*)sqlite3_column_text(stmt, i);
 					if (txt)
-						res.fieldlists[res.rows][i] = SQLEntry(txt);
+						res.fieldlists[res.rows][i] = SQL::Field(txt);
 				}
 				res.rows++;
 			}
@@ -146,7 +173,7 @@ class SQLConn : public SQLProvider
 			}
 			else
 			{
-				SQLerror error(SQL_QREPLY_FAIL, sqlite3_errmsg(conn));
+				SQL::Error error(SQL::QREPLY_FAIL, sqlite3_errmsg(conn));
 				query->OnError(error);
 				break;
 			}
@@ -154,13 +181,14 @@ class SQLConn : public SQLProvider
 		sqlite3_finalize(stmt);
 	}
 
-	virtual void submit(SQLQuery* query, const std::string& q)
+	void Submit(SQL::Query* query, const std::string& q) CXX11_OVERRIDE
 	{
+		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Executing SQLite3 query: " + q);
 		Query(query, q);
 		delete query;
 	}
 
-	virtual void submit(SQLQuery* query, const std::string& q, const ParamL& p)
+	void Submit(SQL::Query* query, const std::string& q, const SQL::ParamList& p) CXX11_OVERRIDE
 	{
 		std::string res;
 		unsigned int param = 0;
@@ -178,10 +206,10 @@ class SQLConn : public SQLProvider
 				}
 			}
 		}
-		submit(query, res);
+		Submit(query, res);
 	}
 
-	virtual void submit(SQLQuery* query, const std::string& q, const ParamM& p)
+	void Submit(SQL::Query* query, const std::string& q, const SQL::ParamMap& p) CXX11_OVERRIDE
 	{
 		std::string res;
 		for(std::string::size_type i = 0; i < q.length(); i++)
@@ -196,7 +224,7 @@ class SQLConn : public SQLProvider
 					field.push_back(q[i++]);
 				i--;
 
-				ParamM::const_iterator it = p.find(field);
+				SQL::ParamMap::const_iterator it = p.find(field);
 				if (it != p.end())
 				{
 					char* escaped = sqlite3_mprintf("%q", it->second.c_str());
@@ -205,29 +233,16 @@ class SQLConn : public SQLProvider
 				}
 			}
 		}
-		submit(query, res);
+		Submit(query, res);
 	}
 };
 
 class ModuleSQLite3 : public Module
 {
- private:
 	ConnMap conns;
 
  public:
-	ModuleSQLite3()
-	{
-	}
-
-	void init()
-	{
-		ReadConf();
-
-		Implementation eventlist[] = { I_OnRehash };
-		ServerInstance->Modules->Attach(eventlist, this, sizeof(eventlist)/sizeof(Implementation));
-	}
-
-	virtual ~ModuleSQLite3()
+	~ModuleSQLite3()
 	{
 		ClearConns();
 	}
@@ -243,13 +258,13 @@ class ModuleSQLite3 : public Module
 		conns.clear();
 	}
 
-	void ReadConf()
+	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
 	{
 		ClearConns();
 		ConfigTagList tags = ServerInstance->Config->ConfTags("database");
 		for(ConfigIter i = tags.first; i != tags.second; i++)
 		{
-			if (i->second->getString("module", "sqlite") != "sqlite")
+			if (!stdalgo::string::equalsci(i->second->getString("module"), "sqlite"))
 				continue;
 			SQLConn* conn = new SQLConn(this, i->second);
 			conns.insert(std::make_pair(i->second->getString("id"), conn));
@@ -257,14 +272,9 @@ class ModuleSQLite3 : public Module
 		}
 	}
 
-	void OnRehash(User* user)
+	Version GetVersion() CXX11_OVERRIDE
 	{
-		ReadConf();
-	}
-
-	Version GetVersion()
-	{
-		return Version("sqlite3 provider", VF_VENDOR);
+		return Version("Provides SQLite3 support", VF_VENDOR);
 	}
 };
 
