@@ -19,36 +19,22 @@
 
 #include "inspircd.h"
 
-/* $ModDesc: Provides support for channel mode +A, allowing /invite freely on a channel and extban A to deny specific users it */
-
-class AllowInvite : public SimpleChannelModeHandler
-{
- public:
-	AllowInvite(Module* Creator) : SimpleChannelModeHandler(Creator, "allowinvite", 'A') { }
-};
-
 class ModuleAllowInvite : public Module
 {
-	AllowInvite ni;
+	SimpleChannelModeHandler ni;
  public:
 
-	ModuleAllowInvite() : ni(this)
+	ModuleAllowInvite()
+		: ni(this, "allowinvite", 'A')
 	{
 	}
 
-	void init()
+	void On005Numeric(std::map<std::string, std::string>& tokens) CXX11_OVERRIDE
 	{
-		ServerInstance->Modules->AddService(ni);
-		Implementation eventlist[] = { I_OnUserPreInvite, I_On005Numeric };
-		ServerInstance->Modules->Attach(eventlist, this, sizeof(eventlist)/sizeof(Implementation));
+		tokens["EXTBAN"].push_back('A');
 	}
 
-	virtual void On005Numeric(std::string &output)
-	{
-		ServerInstance->AddExtBanChar('A');
-	}
-
-	virtual ModResult OnUserPreInvite(User* user,User* dest,Channel* channel, time_t timeout)
+	ModResult OnUserPreInvite(User* user,User* dest,Channel* channel, time_t timeout) CXX11_OVERRIDE
 	{
 		if (IS_LOCAL(user))
 		{
@@ -56,10 +42,10 @@ class ModuleAllowInvite : public Module
 			if (res == MOD_RES_DENY)
 			{
 				// Matching extban, explicitly deny /invite
-				user->WriteNumeric(ERR_CHANOPRIVSNEEDED, "%s %s :You are banned from using INVITE", user->nick.c_str(), channel->name.c_str());
+				user->WriteNumeric(ERR_CHANOPRIVSNEEDED, channel->name, "You are banned from using INVITE");
 				return res;
 			}
-			if (channel->IsModeSet('A') || res == MOD_RES_ALLOW)
+			if (channel->IsModeSet(ni) || res == MOD_RES_ALLOW)
 			{
 				// Explicitly allow /invite
 				return MOD_RES_ALLOW;
@@ -69,13 +55,9 @@ class ModuleAllowInvite : public Module
 		return MOD_RES_PASSTHRU;
 	}
 
-	virtual ~ModuleAllowInvite()
+	Version GetVersion() CXX11_OVERRIDE
 	{
-	}
-
-	virtual Version GetVersion()
-	{
-		return Version("Provides support for channel mode +A, allowing /invite freely on a channel and extban A to deny specific users it",VF_VENDOR);
+		return Version("Provides channel mode +A to allow /INVITE freely on a channel, and extban 'A' to deny specific users it", VF_VENDOR);
 	}
 };
 
