@@ -21,38 +21,35 @@
 
 #include "inspircd.h"
 
-/* $ModDesc: Provides support for the CHGHOST command */
-
 /** Handle /CHGHOST
  */
 class CommandChghost : public Command
 {
- private:
-	char* hostmap;
  public:
-	CommandChghost(Module* Creator, char* hmap) : Command(Creator,"CHGHOST", 2), hostmap(hmap)
+	std::bitset<UCHAR_MAX> hostmap;
+
+	CommandChghost(Module* Creator)
+		: Command(Creator,"CHGHOST", 2)
 	{
 		allow_empty_last_param = false;
 		flags_needed = 'o';
-		syntax = "<nick> <newhost>";
-		TRANSLATE3(TR_NICK, TR_TEXT, TR_END);
+		syntax = "<nick> <host>";
+		TRANSLATE2(TR_NICK, TR_TEXT);
 	}
 
-	CmdResult Handle(const std::vector<std::string> &parameters, User *user)
+	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE
 	{
-		const char* x = parameters[1].c_str();
-
-		if (parameters[1].length() > 63)
+		if (parameters[1].length() > ServerInstance->Config->Limits.MaxHost)
 		{
-			user->WriteServ("NOTICE %s :*** CHGHOST: Host too long", user->nick.c_str());
+			user->WriteNotice("*** CHGHOST: Host too long");
 			return CMD_FAILURE;
 		}
 
-		for (; *x; x++)
+		for (std::string::const_iterator x = parameters[1].begin(); x != parameters[1].end(); x++)
 		{
-			if (!hostmap[(unsigned char)*x])
+			if (!hostmap.test(static_cast<unsigned char>(*x)))
 			{
-				user->WriteServ("NOTICE "+user->nick+" :*** CHGHOST: Invalid characters in hostname");
+				user->WriteNotice("*** CHGHOST: Invalid characters in hostname");
 				return CMD_FAILURE;
 			}
 		}
@@ -60,30 +57,27 @@ class CommandChghost : public Command
 		User* dest = ServerInstance->FindNick(parameters[0]);
 
 		// Allow services to change the host of unregistered users
-		if ((!dest) || ((dest->registered != REG_ALL) && (!ServerInstance->ULine(user->server))))
+		if ((!dest) || ((dest->registered != REG_ALL) && (!user->server->IsULine())))
 		{
-			user->WriteNumeric(ERR_NOSUCHNICK, "%s %s :No such nick/channel", user->nick.c_str(), parameters[0].c_str());
+			user->WriteNumeric(Numerics::NoSuchNick(parameters[0]));
 			return CMD_FAILURE;
 		}
 
 		if (IS_LOCAL(dest))
 		{
-			if ((dest->ChangeDisplayedHost(parameters[1].c_str())) && (!ServerInstance->ULine(user->server)))
+			if ((dest->ChangeDisplayedHost(parameters[1])) && (!user->server->IsULine()))
 			{
 				// fix by brain - ulines set hosts silently
-				ServerInstance->SNO->WriteGlobalSno('a', user->nick+" used CHGHOST to make the displayed host of "+dest->nick+" become "+dest->dhost);
+				ServerInstance->SNO->WriteGlobalSno('a', user->nick+" used CHGHOST to make the displayed host of "+dest->nick+" become "+dest->GetDisplayedHost());
 			}
 		}
 
 		return CMD_SUCCESS;
 	}
 
-	RouteDescriptor GetRouting(User* user, const std::vector<std::string>& parameters)
+	RouteDescriptor GetRouting(User* user, const Params& parameters) CXX11_OVERRIDE
 	{
-		User* dest = ServerInstance->FindNick(parameters[0]);
-		if (dest)
-			return ROUTE_OPT_UCAST(dest->server);
-		return ROUTE_LOCALONLY;
+		return ROUTE_OPT_UCAST(parameters[0]);
 	}
 };
 
@@ -91,38 +85,26 @@ class CommandChghost : public Command
 class ModuleChgHost : public Module
 {
 	CommandChghost cmd;
-	char hostmap[256];
+
  public:
-	ModuleChgHost() : cmd(this, hostmap)
+	ModuleChgHost()
+		: cmd(this)
 	{
 	}
 
-	void init()
-	{
-		OnRehash(NULL);
-		ServerInstance->Modules->AddService(cmd);
-		Implementation eventlist[] = { I_OnRehash };
-		ServerInstance->Modules->Attach(eventlist, this, sizeof(eventlist)/sizeof(Implementation));
-	}
-
-	void OnRehash(User* user)
+	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
 	{
 		std::string hmap = ServerInstance->Config->ConfValue("hostname")->getString("charmap", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz.-_/0123456789");
 
-		memset(hostmap, 0, sizeof(hostmap));
+		cmd.hostmap.reset();
 		for (std::string::iterator n = hmap.begin(); n != hmap.end(); n++)
-			hostmap[(unsigned char)*n] = 1;
+			cmd.hostmap.set(static_cast<unsigned char>(*n));
 	}
 
-	~ModuleChgHost()
+	Version GetVersion() CXX11_OVERRIDE
 	{
+		return Version("Provides the CHGHOST command", VF_OPTCOMMON | VF_VENDOR);
 	}
-
-	Version GetVersion()
-	{
-		return Version("Provides support for the CHGHOST command", VF_OPTCOMMON | VF_VENDOR);
-	}
-
 };
 
 MODULE_INIT(ModuleChgHost)
