@@ -22,69 +22,32 @@
 
 #include "inspircd.h"
 
-/* $ModDesc: Sets (and unsets) modes on users when they connect */
-
 class ModuleModesOnConnect : public Module
 {
  public:
-	void init()
-	{
-		ServerInstance->Modules->Attach(I_OnUserConnect, this);
-	}
-
-	void Prioritize()
-	{
-		// for things like +x on connect, important, otherwise we have to resort to config order (bleh) -- w00t
-		ServerInstance->Modules->SetPriority(this, I_OnUserConnect, PRIORITY_FIRST);
-	}
-
-	virtual ~ModuleModesOnConnect()
-	{
-	}
-
-	virtual Version GetVersion()
+	Version GetVersion() CXX11_OVERRIDE
 	{
 		return Version("Sets (and unsets) modes on users when they connect", VF_VENDOR);
 	}
 
-	virtual void OnUserConnect(LocalUser* user)
+	void OnUserConnect(LocalUser* user) CXX11_OVERRIDE
 	{
-		// Backup and zero out the disabled usermodes, so that we can override them here.
-		char save[64];
-		memcpy(save, ServerInstance->Config->DisabledUModes,
-				sizeof(ServerInstance->Config->DisabledUModes));
-		memset(ServerInstance->Config->DisabledUModes, 0, 64);
-
 		ConfigTag* tag = user->MyClass->config;
 		std::string ThisModes = tag->getString("modes");
 		if (!ThisModes.empty())
 		{
 			std::string buf;
-			std::stringstream ss(ThisModes);
+			irc::spacesepstream ss(ThisModes);
 
-			std::vector<std::string> tokens;
+			CommandBase::Params modes;
+			modes.push_back(user->nick);
 
 			// split ThisUserModes into modes and mode params
-			while (ss >> buf)
-				tokens.push_back(buf);
+			while (ss.GetToken(buf))
+				modes.push_back(buf);
 
-			std::vector<std::string> modes;
-			modes.push_back(user->nick);
-			modes.push_back(tokens[0]);
-
-			if (tokens.size() > 1)
-			{
-				// process mode params
-				for (unsigned int k = 1; k < tokens.size(); k++)
-				{
-					modes.push_back(tokens[k]);
-				}
-			}
-
-			ServerInstance->Parser->CallHandler("MODE", modes, user);
+			ServerInstance->Parser.CallHandler("MODE", modes, user);
 		}
-
-		memcpy(ServerInstance->Config->DisabledUModes, save, 64);
 	}
 };
 
