@@ -21,13 +21,8 @@
  */
 
 
-/* $ModDesc: Allows for MD5 encrypted oper passwords */
-
 #include "inspircd.h"
-#ifdef HAS_STDINT
-#include <stdint.h>
-#endif
-#include "hash.h"
+#include "modules/hash.h"
 
 /* The four core functions - F1 is optimized somewhat */
 #define F1(x, y, z) (z ^ (x & (y ^ z)))
@@ -39,10 +34,6 @@
 #define MD5STEP(f,w,x,y,z,in,s) \
 	(w += f(x,y,z) + in, w = (w<<s | w>>(32-s)) + x)
 
-#ifndef HAS_STDINT
-typedef unsigned int uint32_t;
-#endif
-
 typedef uint32_t word32; /* NOT unsigned long. We don't support 16 bit platforms, anyway. */
 typedef unsigned char byte;
 
@@ -70,23 +61,13 @@ class MD5Provider : public HashProvider
 		} while (--words);
 	}
 
-	void MD5Init(MD5Context *ctx, unsigned int* ikey = NULL)
+	void MD5Init(MD5Context *ctx)
 	{
 		/* These are the defaults for md5 */
-		if (!ikey)
-		{
-			ctx->buf[0] = 0x67452301;
-			ctx->buf[1] = 0xefcdab89;
-			ctx->buf[2] = 0x98badcfe;
-			ctx->buf[3] = 0x10325476;
-		}
-		else
-		{
-			ctx->buf[0] = ikey[0];
-			ctx->buf[1] = ikey[1];
-			ctx->buf[2] = ikey[2];
-			ctx->buf[3] = ikey[3];
-		}
+		ctx->buf[0] = 0x67452301;
+		ctx->buf[1] = 0xefcdab89;
+		ctx->buf[2] = 0x98badcfe;
+		ctx->buf[3] = 0x10325476;
 
 		ctx->bytes[0] = 0;
 		ctx->bytes[1] = 0;
@@ -245,44 +226,23 @@ class MD5Provider : public HashProvider
 	}
 
 
-	void MyMD5(void *dest, void *orig, int len, unsigned int* ikey)
+	void MyMD5(void *dest, void *orig, int len)
 	{
 		MD5Context context;
-		MD5Init(&context, ikey);
+		MD5Init(&context);
 		MD5Update(&context, (const unsigned char*)orig, len);
 		MD5Final((unsigned char*)dest, &context);
 	}
 
-
-	void GenHash(const char* src, char* dest, const char* xtab, unsigned int* ikey, size_t srclen)
-	{
-		unsigned char bytes[16];
-
-		MyMD5((char*)bytes, (void*)src, srclen, ikey);
-
-		for (int i = 0; i < 16; i++)
-		{
-			*dest++ = xtab[bytes[i] / 16];
-			*dest++ = xtab[bytes[i] % 16];
-		}
-		*dest++ = 0;
-	}
  public:
-	std::string sum(const std::string& data)
+	std::string GenerateRaw(const std::string& data) CXX11_OVERRIDE
 	{
 		char res[16];
-		MyMD5(res, (void*)data.data(), data.length(), NULL);
+		MyMD5(res, (void*)data.data(), data.length());
 		return std::string(res, 16);
 	}
 
-	std::string sumIV(unsigned int* IV, const char* HexMap, const std::string &sdata)
-	{
-		char res[33];
-		GenHash(sdata.data(), res, HexMap, IV, sdata.length());
-		return res;
-	}
-
-	MD5Provider(Module* parent) : HashProvider(parent, "hash/md5", 16, 64) {}
+	MD5Provider(Module* parent) : HashProvider(parent, "md5", 16, 64) {}
 };
 
 class ModuleMD5 : public Module
@@ -291,10 +251,9 @@ class ModuleMD5 : public Module
  public:
 	ModuleMD5() : md5(this)
 	{
-		ServerInstance->Modules->AddService(md5);
 	}
 
-	Version GetVersion()
+	Version GetVersion() CXX11_OVERRIDE
 	{
 		return Version("Implements MD5 hashing",VF_VENDOR);
 	}
