@@ -20,45 +20,29 @@
 
 
 #include "inspircd.h"
+#include "modules/ctctags.h"
 
-/* $ModDesc: Forbids users from messaging each other. Users may still message opers and opers may message other opers. */
-
-
-class ModuleRestrictMsg : public Module
+class ModuleRestrictMsg
+	: public Module
+	, public CTCTags::EventListener
 {
  private:
-	bool uline;
-
- public:
-
-	void init()
+	ModResult HandleMessage(User* user, const MessageTarget& target)
 	{
-		OnRehash(NULL);
-		Implementation eventlist[] = { I_OnRehash, I_OnUserPreMessage, I_OnUserPreNotice };
-		ServerInstance->Modules->Attach(eventlist, this, sizeof(eventlist)/sizeof(Implementation));
-	}
-
-	void OnRehash(User*)
-	{
-		uline = ServerInstance->Config->ConfValue("restrictmsg")->getBool("uline", false);
-	}
-
-	virtual ModResult OnUserPreMessage(User* user,void* dest,int target_type, std::string &text, char status, CUList &exempt_list)
-	{
-		if ((target_type == TYPE_USER) && (IS_LOCAL(user)))
+		if ((target.type == MessageTarget::TYPE_USER) && (IS_LOCAL(user)))
 		{
-			User* u = (User*)dest;
+			User* u = target.Get<User>();
 
 			// message allowed if:
 			// (1) the sender is opered
 			// (2) the recipient is opered
 			// (3) the recipient is on a ulined server
 			// anything else, blocked.
-			if (IS_OPER(u) || IS_OPER(user) || (uline && ServerInstance->ULine(u->server)))
+			if (u->IsOper() || user->IsOper() || u->server->IsULine())
 			{
 				return MOD_RES_PASSTHRU;
 			}
-			user->WriteNumeric(ERR_CANTSENDTOUSER, "%s %s :You are not permitted to send private messages to this user",user->nick.c_str(),u->nick.c_str());
+			user->WriteNumeric(ERR_CANTSENDTOUSER, u->nick, "You are not permitted to send private messages to this user");
 			return MOD_RES_DENY;
 		}
 
@@ -66,18 +50,25 @@ class ModuleRestrictMsg : public Module
 		return MOD_RES_PASSTHRU;
 	}
 
-	virtual ModResult OnUserPreNotice(User* user,void* dest,int target_type, std::string &text, char status, CUList &exempt_list)
-	{
-		return this->OnUserPreMessage(user,dest,target_type,text,status,exempt_list);
-	}
-
-	virtual ~ModuleRestrictMsg()
+ public:
+	ModuleRestrictMsg()
+		: CTCTags::EventListener(this)
 	{
 	}
 
-	virtual Version GetVersion()
+	ModResult OnUserPreMessage(User* user, const MessageTarget& target, MessageDetails& details) CXX11_OVERRIDE
 	{
-		return Version("Forbids users from messaging each other. Users may still message opers and opers may message other opers.",VF_VENDOR);
+		return HandleMessage(user, target);
+	}
+
+	ModResult OnUserPreTagMessage(User* user, const MessageTarget& target, CTCTags::TagMessageDetails& details) CXX11_OVERRIDE
+	{
+		return HandleMessage(user, target);
+	}
+
+	Version GetVersion() CXX11_OVERRIDE
+	{
+		return Version("Forbids users from messaging each other, but users may still message opers and opers may message other opers", VF_VENDOR);
 	}
 };
 
