@@ -21,43 +21,40 @@
 
 #include "inspircd.h"
 
-/* $ModDesc: Provides support for the SETHOST command */
-
 /** Handle /SETHOST
  */
 class CommandSethost : public Command
 {
- private:
-	char* hostmap;
  public:
-	CommandSethost(Module* Creator, char* hmap) : Command(Creator,"SETHOST", 1), hostmap(hmap)
+	std::bitset<UCHAR_MAX> hostmap;
+
+	CommandSethost(Module* Creator)
+		: Command(Creator,"SETHOST", 1)
 	{
 		allow_empty_last_param = false;
-		flags_needed = 'o'; syntax = "<new-hostname>";
-		TRANSLATE2(TR_TEXT, TR_END);
+		flags_needed = 'o'; syntax = "<host>";
 	}
 
-	CmdResult Handle (const std::vector<std::string>& parameters, User *user)
+	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE
 	{
-		size_t len = 0;
-		for (std::string::const_iterator x = parameters[0].begin(); x != parameters[0].end(); x++, len++)
+		if (parameters[0].length() > ServerInstance->Config->Limits.MaxHost)
 		{
-			if (!hostmap[(const unsigned char)*x])
+			user->WriteNotice("*** SETHOST: Host too long");
+			return CMD_FAILURE;
+		}
+
+		for (std::string::const_iterator x = parameters[0].begin(); x != parameters[0].end(); x++)
+		{
+			if (!hostmap.test(static_cast<unsigned char>(*x)))
 			{
-				user->WriteServ("NOTICE "+user->nick+" :*** SETHOST: Invalid characters in hostname");
+				user->WriteNotice("*** SETHOST: Invalid characters in hostname");
 				return CMD_FAILURE;
 			}
 		}
 
-		if (len > 64)
+		if (user->ChangeDisplayedHost(parameters[0]))
 		{
-			user->WriteServ("NOTICE %s :*** SETHOST: Host too long",user->nick.c_str());
-			return CMD_FAILURE;
-		}
-
-		if (user->ChangeDisplayedHost(parameters[0].c_str()))
-		{
-			ServerInstance->SNO->WriteGlobalSno('a', user->nick+" used SETHOST to change their displayed host to "+user->dhost);
+			ServerInstance->SNO->WriteGlobalSno('a', user->nick+" used SETHOST to change their displayed host to "+user->GetDisplayedHost());
 			return CMD_SUCCESS;
 		}
 
@@ -69,39 +66,26 @@ class CommandSethost : public Command
 class ModuleSetHost : public Module
 {
 	CommandSethost cmd;
-	char hostmap[256];
+
  public:
 	ModuleSetHost()
-		: cmd(this, hostmap)
+		: cmd(this)
 	{
 	}
 
-	void init()
-	{
-		OnRehash(NULL);
-		ServerInstance->Modules->AddService(cmd);
-		Implementation eventlist[] = { I_OnRehash };
-		ServerInstance->Modules->Attach(eventlist, this, sizeof(eventlist)/sizeof(Implementation));
-	}
-
-	void OnRehash(User* user)
+	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
 	{
 		std::string hmap = ServerInstance->Config->ConfValue("hostname")->getString("charmap", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz.-_/0123456789");
 
-		memset(hostmap, 0, sizeof(hostmap));
+		cmd.hostmap.reset();
 		for (std::string::iterator n = hmap.begin(); n != hmap.end(); n++)
-			hostmap[(unsigned char)*n] = 1;
+			cmd.hostmap.set(static_cast<unsigned char>(*n));
 	}
 
-	virtual ~ModuleSetHost()
+	Version GetVersion() CXX11_OVERRIDE
 	{
+		return Version("Provides the SETHOST command", VF_VENDOR);
 	}
-
-	virtual Version GetVersion()
-	{
-		return Version("Provides support for the SETHOST command", VF_VENDOR);
-	}
-
 };
 
 MODULE_INIT(ModuleSetHost)
