@@ -21,25 +21,29 @@
 
 #include "inspircd.h"
 
-/* $ModDesc: Allows opers to set their idle time */
+enum
+{
+	// InspIRCd-specific.
+	ERR_INVALIDIDLETIME = 948,
+	RPL_IDLETIMESET = 944
+};
 
 /** Handle /SETIDLE
  */
-class CommandSetidle : public Command
+class CommandSetidle : public SplitCommand
 {
  public:
-	CommandSetidle(Module* Creator) : Command(Creator,"SETIDLE", 1)
+	CommandSetidle(Module* Creator) : SplitCommand(Creator,"SETIDLE", 1)
 	{
 		flags_needed = 'o'; syntax = "<duration>";
-		TRANSLATE2(TR_TEXT, TR_END);
 	}
 
-	CmdResult Handle (const std::vector<std::string>& parameters, User *user)
+	CmdResult HandleLocal(LocalUser* user, const Params& parameters) CXX11_OVERRIDE
 	{
-		time_t idle = ServerInstance->Duration(parameters[0]);
-		if (idle < 1)
+		unsigned long idle;
+		if (!InspIRCd::Duration(parameters[0], idle))
 		{
-			user->WriteNumeric(948, "%s :Invalid idle time.",user->nick.c_str());
+			user->WriteNumeric(ERR_INVALIDIDLETIME, "Invalid idle time.");
 			return CMD_FAILURE;
 		}
 		user->idle_lastmsg = (ServerInstance->Time() - idle);
@@ -47,7 +51,7 @@ class CommandSetidle : public Command
 		if (user->signon > user->idle_lastmsg)
 			user->signon = user->idle_lastmsg;
 		ServerInstance->SNO->WriteToSnoMask('a', user->nick+" used SETIDLE to set their idle time to "+ConvToStr(idle)+" seconds");
-		user->WriteNumeric(944, "%s :Idle time set.",user->nick.c_str());
+		user->WriteNumeric(RPL_IDLETIMESET, "Idle time set.");
 
 		return CMD_SUCCESS;
 	}
@@ -63,18 +67,9 @@ class ModuleSetIdle : public Module
 	{
 	}
 
-	void init()
+	Version GetVersion() CXX11_OVERRIDE
 	{
-		ServerInstance->Modules->AddService(cmd);
-	}
-
-	virtual ~ModuleSetIdle()
-	{
-	}
-
-	virtual Version GetVersion()
-	{
-		return Version("Allows opers to set their idle time", VF_VENDOR);
+		return Version("Provides the SETIDLE command, allows opers to set their idle time", VF_VENDOR);
 	}
 };
 
