@@ -21,31 +21,31 @@
 
 #include "inspircd.h"
 
-/* $ModDesc: Provides support for the SETNAME command */
-
 
 
 class CommandSetname : public Command
 {
  public:
+	bool notifyopers;
 	CommandSetname(Module* Creator) : Command(Creator,"SETNAME", 1, 1)
 	{
 		allow_empty_last_param = false;
-		syntax = "<new-gecos>";
-		TRANSLATE2(TR_TEXT, TR_END);
+		syntax = ":<realname>";
 	}
 
-	CmdResult Handle (const std::vector<std::string>& parameters, User *user)
+	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE
 	{
-		if (parameters[0].size() > ServerInstance->Config->Limits.MaxGecos)
+		if (parameters[0].size() > ServerInstance->Config->Limits.MaxReal)
 		{
-			user->WriteServ("NOTICE %s :*** SETNAME: GECOS too long", user->nick.c_str());
+			user->WriteNotice("*** SETNAME: Real name is too long");
 			return CMD_FAILURE;
 		}
 
-		if (user->ChangeName(parameters[0].c_str()))
+		if (user->ChangeRealName(parameters[0]))
 		{
-			ServerInstance->SNO->WriteGlobalSno('a', "%s used SETNAME to change their GECOS to '%s'", user->nick.c_str(), parameters[0].c_str());
+			if (notifyopers)
+				ServerInstance->SNO->WriteGlobalSno('a', "%s used SETNAME to change their real name to '%s'",
+					user->nick.c_str(), parameters[0].c_str());
 		}
 
 		return CMD_SUCCESS;
@@ -62,18 +62,21 @@ class ModuleSetName : public Module
 	{
 	}
 
-	void init()
+	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
 	{
-		ServerInstance->Modules->AddService(cmd);
+		ConfigTag* tag = ServerInstance->Config->ConfValue("setname");
+
+		// Whether the module should only be usable by server operators.
+		bool operonly = tag->getBool("operonly");
+		cmd.flags_needed = operonly ? 'o' : 0;
+
+		// Whether a snotice should be sent out when a user changes their real name.
+		cmd.notifyopers = tag->getBool("notifyopers", !operonly);
 	}
 
-	virtual ~ModuleSetName()
+	Version GetVersion() CXX11_OVERRIDE
 	{
-	}
-
-	virtual Version GetVersion()
-	{
-		return Version("Provides support for the SETNAME command", VF_VENDOR);
+		return Version("Provides the SETNAME command", VF_VENDOR);
 	}
 };
 
