@@ -22,54 +22,9 @@
 
 #include "inspircd.h"
 #include "xline.h"
+#include "modules/shun.h"
+#include "modules/stats.h"
 
-/* $ModDesc: Provides the /SHUN command, which stops a user from executing all except configured commands. */
-
-class Shun : public XLine
-{
-public:
-	std::string matchtext;
-
-	Shun(time_t s_time, long d, const std::string& src, const std::string& re, const std::string& shunmask)
-		: XLine(s_time, d, src, re, "SHUN")
-		, matchtext(shunmask)
-	{
-	}
-
-	~Shun()
-	{
-	}
-
-	bool Matches(User *u)
-	{
-		// E: overrides shun
-		if (u->exempt)
-			return false;
-
-		if (InspIRCd::Match(u->GetFullHost(), matchtext) || InspIRCd::Match(u->GetFullRealHost(), matchtext) || InspIRCd::Match(u->nick+"!"+u->ident+"@"+u->GetIPString(), matchtext))
-			return true;
-
-		return false;
-	}
-
-	bool Matches(const std::string &s)
-	{
-		if (matchtext == s)
-			return true;
-		return false;
-	}
-
-	void DisplayExpiry()
-	{
-		ServerInstance->SNO->WriteToSnoMask('x',"Removing expired shun %s (set by %s %ld seconds ago)",
-			this->matchtext.c_str(), this->source.c_str(), (long int)(ServerInstance->Time() - this->set_time));
-	}
-
-	const char* Displayable()
-	{
-		return matchtext.c_str();
-	}
-};
 
 /** An XLineFactory specialized to generate shun pointers
  */
@@ -80,12 +35,12 @@ class ShunFactory : public XLineFactory
 
 	/** Generate a shun
  	*/
-	XLine* Generate(time_t set_time, long duration, std::string source, std::string reason, std::string xline_specific_mask)
+	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
 	{
 		return new Shun(set_time, duration, source, reason, xline_specific_mask);
 	}
 
-	bool AutoApplyToUserList(XLine *x)
+	bool AutoApplyToUserList(XLine* x) CXX11_OVERRIDE
 	{
 		return false;
 	}
@@ -98,44 +53,50 @@ class CommandShun : public Command
  public:
 	CommandShun(Module* Creator) : Command(Creator, "SHUN", 1, 3)
 	{
-		flags_needed = 'o'; this->syntax = "<nick!user@hostmask> [<shun-duration>] :<reason>";
+		flags_needed = 'o'; this->syntax = "<nick!user@host> [<duration> :<reason>]";
 	}
 
-	CmdResult Handle(const std::vector<std::string>& parameters, User *user)
+	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE
 	{
 		/* syntax: SHUN nick!user@host time :reason goes here */
 		/* 'time' is a human-readable timestring, like 2d3h2s. */
 
 		std::string target = parameters[0];
-		
+
 		User *find = ServerInstance->FindNick(target);
 		if ((find) && (find->registered == REG_ALL))
 			target = std::string("*!*@") + find->GetIPString();
 
 		if (parameters.size() == 1)
 		{
-			if (ServerInstance->XLines->DelLine(parameters[0].c_str(), "SHUN", user))
+			std::string reason;
+
+			if (ServerInstance->XLines->DelLine(parameters[0].c_str(), "SHUN", reason, user))
 			{
-				ServerInstance->SNO->WriteToSnoMask('x', "%s removed SHUN on %s", user->nick.c_str(), parameters[0].c_str());
+				ServerInstance->SNO->WriteToSnoMask('x', "%s removed SHUN on %s: %s", user->nick.c_str(), parameters[0].c_str(), reason.c_str());
 			}
-			else if (ServerInstance->XLines->DelLine(target.c_str(), "SHUN", user))
+			else if (ServerInstance->XLines->DelLine(target.c_str(), "SHUN", reason, user))
 			{
-				ServerInstance->SNO->WriteToSnoMask('x',"%s removed SHUN on %s",user->nick.c_str(),target.c_str());
+				ServerInstance->SNO->WriteToSnoMask('x', "%s removed SHUN on %s: %s", user->nick.c_str(), target.c_str(), reason.c_str());
 			}
 			else
 			{
-				user->WriteServ("NOTICE %s :*** Shun %s not found in list, try /stats H.", user->nick.c_str(), parameters[0].c_str());
+				user->WriteNotice("*** Shun " + parameters[0] + " not found on the list.");
 				return CMD_FAILURE;
 			}
 		}
 		else
 		{
 			// Adding - XXX todo make this respect <insane> tag perhaps..
-			long duration;
+			unsigned long duration;
 			std::string expr;
 			if (parameters.size() > 2)
 			{
-				duration = ServerInstance->Duration(parameters[1]);
+				if (!InspIRCd::Duration(parameters[1], duration))
+				{
+					user->WriteNotice("*** Invalid duration for SHUN.");
+					return CMD_FAILURE;
+				}
 				expr = parameters[2];
 			}
 			else
@@ -149,28 +110,27 @@ class CommandShun : public Command
 			{
 				if (!duration)
 				{
-					ServerInstance->SNO->WriteToSnoMask('x',"%s added permanent SHUN for %s: %s",
+					ServerInstance->SNO->WriteToSnoMask('x', "%s added permanent SHUN for %s: %s",
 						user->nick.c_str(), target.c_str(), expr.c_str());
 				}
 				else
 				{
-					time_t c_requires_crap = duration + ServerInstance->Time();
-					std::string timestr = ServerInstance->TimeString(c_requires_crap);
-					ServerInstance->SNO->WriteToSnoMask('x', "%s added timed SHUN for %s to expire on %s: %s",
-						user->nick.c_str(), target.c_str(), timestr.c_str(), expr.c_str());
+					ServerInstance->SNO->WriteToSnoMask('x', "%s added timed SHUN for %s, expires in %s (on %s): %s",
+						user->nick.c_str(), target.c_str(), InspIRCd::DurationString(duration).c_str(),
+						InspIRCd::TimeString(ServerInstance->Time() + duration).c_str(), expr.c_str());
 				}
 			}
 			else
 			{
 				delete r;
-				user->WriteServ("NOTICE %s :*** Shun for %s already exists", user->nick.c_str(), target.c_str());
+				user->WriteNotice("*** Shun for " + target + " already exists.");
 				return CMD_FAILURE;
 			}
 		}
 		return CMD_SUCCESS;
 	}
 
-	RouteDescriptor GetRouting(User* user, const std::vector<std::string>& parameters)
+	RouteDescriptor GetRouting(User* user, const Params& parameters) CXX11_OVERRIDE
 	{
 		if (IS_LOCAL(user))
 			return ROUTE_LOCALONLY; // spanningtree will send ADDLINE
@@ -179,51 +139,48 @@ class CommandShun : public Command
 	}
 };
 
-class ModuleShun : public Module
+class ModuleShun : public Module, public Stats::EventListener
 {
 	CommandShun cmd;
 	ShunFactory f;
-	std::set<std::string> ShunEnabledCommands;
+	insp::flat_set<std::string> ShunEnabledCommands;
 	bool NotifyOfShun;
 	bool affectopers;
 
  public:
-	ModuleShun() : cmd(this)
+	ModuleShun()
+		: Stats::EventListener(this)
+		, cmd(this)
 	{
 	}
 
-	void init()
+	void init() CXX11_OVERRIDE
 	{
 		ServerInstance->XLines->RegisterFactory(&f);
-		ServerInstance->Modules->AddService(cmd);
-
-		Implementation eventlist[] = { I_OnStats, I_OnPreCommand, I_OnRehash };
-		ServerInstance->Modules->Attach(eventlist, this, sizeof(eventlist)/sizeof(Implementation));
-		OnRehash(NULL);
 	}
 
-	virtual ~ModuleShun()
+	~ModuleShun()
 	{
 		ServerInstance->XLines->DelAll("SHUN");
 		ServerInstance->XLines->UnregisterFactory(&f);
 	}
 
-	void Prioritize()
+	void Prioritize() CXX11_OVERRIDE
 	{
 		Module* alias = ServerInstance->Modules->Find("m_alias.so");
-		ServerInstance->Modules->SetPriority(this, I_OnPreCommand, PRIORITY_BEFORE, &alias);
+		ServerInstance->Modules->SetPriority(this, I_OnPreCommand, PRIORITY_BEFORE, alias);
 	}
 
-	virtual ModResult OnStats(char symbol, User* user, string_list& out)
+	ModResult OnStats(Stats::Context& stats) CXX11_OVERRIDE
 	{
-		if (symbol != 'H')
+		if (stats.GetSymbol() != 'H')
 			return MOD_RES_PASSTHRU;
 
-		ServerInstance->XLines->InvokeStats("SHUN", 223, user, out);
+		ServerInstance->XLines->InvokeStats("SHUN", 223, stats);
 		return MOD_RES_DENY;
 	}
 
-	virtual void OnRehash(User* user)
+	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
 	{
 		ConfigTag* tag = ServerInstance->Config->ConfValue("shun");
 		std::string cmds = tag->getString("enabledcommands");
@@ -234,10 +191,10 @@ class ModuleShun : public Module
 
 		ShunEnabledCommands.clear();
 
-		std::stringstream dcmds(cmds);
+		irc::spacesepstream dcmds(cmds);
 		std::string thiscmd;
 
-		while (dcmds >> thiscmd)
+		while (dcmds.GetToken(thiscmd))
 		{
 			ShunEnabledCommands.insert(thiscmd);
 		}
@@ -246,7 +203,7 @@ class ModuleShun : public Module
 		affectopers = tag->getBool("affectopers", false);
 	}
 
-	virtual ModResult OnPreCommand(std::string &command, std::vector<std::string>& parameters, LocalUser* user, bool validated, const std::string &original_line)
+	ModResult OnPreCommand(std::string& command, CommandBase::Params& parameters, LocalUser* user, bool validated) CXX11_OVERRIDE
 	{
 		if (validated)
 			return MOD_RES_PASSTHRU;
@@ -257,18 +214,16 @@ class ModuleShun : public Module
 			return MOD_RES_PASSTHRU;
 		}
 
-		if (!affectopers && IS_OPER(user))
+		if (!affectopers && user->IsOper())
 		{
 			/* Don't do anything if the user is an operator and affectopers isn't set */
 			return MOD_RES_PASSTHRU;
 		}
 
-		std::set<std::string>::iterator i = ShunEnabledCommands.find(command);
-
-		if (i == ShunEnabledCommands.end())
+		if (!ShunEnabledCommands.count(command))
 		{
 			if (NotifyOfShun)
-				user->WriteServ("NOTICE %s :*** Command %s not processed, as you have been blocked from issuing commands (SHUN)", user->nick.c_str(), command.c_str());
+				user->WriteNotice("*** Command " + command + " not processed, as you have been blocked from issuing commands (SHUN)");
 			return MOD_RES_DENY;
 		}
 
@@ -287,11 +242,10 @@ class ModuleShun : public Module
 		return MOD_RES_PASSTHRU;
 	}
 
-	virtual Version GetVersion()
+	Version GetVersion() CXX11_OVERRIDE
 	{
-		return Version("Provides the /SHUN command, which stops a user from executing all except configured commands.",VF_VENDOR|VF_COMMON);
+		return Version("Provides the SHUN command, which stops a user from executing all except configured commands", VF_VENDOR|VF_COMMON);
 	}
 };
 
 MODULE_INIT(ModuleShun)
-
