@@ -21,32 +21,32 @@
 
 #include "main.h"
 #include "utils.h"
-#include "treeserver.h"
-#include "treesocket.h"
+#include "commands.h"
 
-bool TreeSocket::Away(const std::string &prefix, parameterlist &params)
+CmdResult CommandAway::HandleRemote(::RemoteUser* u, Params& params)
 {
-	User* u = ServerInstance->FindNick(prefix);
-	if ((!u) || (IS_SERVER(u)))
-		return true;
-	if (params.size())
+	if (!params.empty())
 	{
-		FOREACH_MOD(I_OnSetAway, OnSetAway(u, params[params.size() - 1]));
-
 		if (params.size() > 1)
-			u->awaytime = atoi(params[0].c_str());
+			u->awaytime = ConvToNum<time_t>(params[0]);
 		else
 			u->awaytime = ServerInstance->Time();
 
-		u->awaymsg = params[params.size() - 1];
-
-		params[params.size() - 1] = ":" + params[params.size() - 1];
+		u->awaymsg = params.back();
+		FOREACH_MOD_CUSTOM(awayevprov, Away::EventListener, OnUserAway, (u));
 	}
 	else
 	{
-		FOREACH_MOD(I_OnSetAway, OnSetAway(u, ""));
+		u->awaytime = 0;
 		u->awaymsg.clear();
+		FOREACH_MOD_CUSTOM(awayevprov, Away::EventListener, OnUserBack, (u));
 	}
-	Utils->DoOneToAllButSender(prefix,"AWAY",params,u->server);
-	return true;
+	return CMD_SUCCESS;
+}
+
+CommandAway::Builder::Builder(User* user)
+	: CmdBuilder(user, "AWAY")
+{
+	if (!user->awaymsg.empty())
+		push_int(user->awaytime).push_last(user->awaymsg);
 }
