@@ -18,23 +18,22 @@
  */
 
 
-#ifndef M_SPANNINGTREE_LINK_H
-#define M_SPANNINGTREE_LINK_H
+#pragma once
 
 class Link : public refcountbase
 {
  public:
 	reference<ConfigTag> tag;
-	irc::string Name;
+	std::string Name;
 	std::string IPAddr;
-	int Port;
+	unsigned int Port;
 	std::string SendPass;
 	std::string RecvPass;
 	std::string Fingerprint;
-	std::string AllowMask;
+	std::vector<std::string> AllowMasks;
 	bool HiddenFromStats;
 	std::string Hook;
-	int Timeout;
+	unsigned int Timeout;
 	std::string Bind;
 	bool Hidden;
 	Link(ConfigTag* Tag) : tag(Tag) {}
@@ -51,5 +50,3 @@ class Autoconnect : public refcountbase
 	int position;
 	Autoconnect(ConfigTag* Tag) : tag(Tag) {}
 };
-
-#endif
