@@ -19,16 +19,10 @@
 
 
 #include "inspircd.h"
-#include "socket.h"
-#include "xline.h"
-#include "socketengine.h"
 
-#include "main.h"
-#include "utils.h"
-#include "treeserver.h"
 #include "commands.h"
 
-CmdResult CommandSVSPart::Handle(const std::vector<std::string>& parameters, User *user)
+CmdResult CommandSVSPart::Handle(User* user, Params& parameters)
 {
 	User* u = ServerInstance->FindUUID(parameters[0]);
 	if (!u)
@@ -46,10 +40,7 @@ CmdResult CommandSVSPart::Handle(const std::vector<std::string>& parameters, Use
 	return CMD_SUCCESS;
 }
 
-RouteDescriptor CommandSVSPart::GetRouting(User* user, const std::vector<std::string>& parameters)
+RouteDescriptor CommandSVSPart::GetRouting(User* user, const Params& parameters)
 {
-	User* u = ServerInstance->FindUUID(parameters[0]);
-	if (u)
-		return ROUTE_OPT_UCAST(u->server);
-	return ROUTE_LOCALONLY;
+	return ROUTE_OPT_UCAST(parameters[0]);
 }
