@@ -22,8 +22,7 @@
 
 #include "inspircd.h"
 #include "xline.h"
-
-/* $ModDesc: Implements SVSHOLD. Like Q:Lines, but can only be added/removed by Services. */
+#include "modules/stats.h"
 
 namespace
 {
@@ -35,44 +34,38 @@ namespace
 class SVSHold : public XLine
 {
 public:
-	irc::string nickname;
+	std::string nickname;
 
-	SVSHold(time_t s_time, long d, const std::string& src, const std::string& re, const std::string& nick)
+	SVSHold(time_t s_time, unsigned long d, const std::string& src, const std::string& re, const std::string& nick)
 		: XLine(s_time, d, src, re, "SVSHOLD")
 	{
-		this->nickname = nick.c_str();
+		this->nickname = nick;
 	}
 
-	~SVSHold()
-	{
-	}
-
-	bool Matches(User *u)
+	bool Matches(User* u) CXX11_OVERRIDE
 	{
 		if (u->nick == nickname)
 			return true;
 		return false;
 	}
 
-	bool Matches(const std::string &s)
+	bool Matches(const std::string& s) CXX11_OVERRIDE
 	{
-		if (nickname == s)
-			return true;
-		return false;
+		return InspIRCd::Match(s, nickname);
 	}
 
-	void DisplayExpiry()
+	void DisplayExpiry() CXX11_OVERRIDE
 	{
 		if (!silent)
 		{
-			ServerInstance->SNO->WriteToSnoMask('x',"Removing expired SVSHOLD %s (set by %s %ld seconds ago)",
-				this->nickname.c_str(), this->source.c_str(), (long int)(ServerInstance->Time() - this->set_time));
+			ServerInstance->SNO->WriteToSnoMask('x', "Removing expired SVSHOLD %s (set by %s %s ago): %s",
+				nickname.c_str(), source.c_str(), InspIRCd::DurationString(ServerInstance->Time() - set_time).c_str(), reason.c_str());
 		}
 	}
 
-	const char* Displayable()
+	const std::string& Displayable() CXX11_OVERRIDE
 	{
-		return nickname.c_str();
+		return nickname;
 	}
 };
 
@@ -83,14 +76,14 @@ class SVSHoldFactory : public XLineFactory
  public:
 	SVSHoldFactory() : XLineFactory("SVSHOLD") { }
 
-	/** Generate a shun
+	/** Generate an SVSHOLD
  	*/
-	XLine* Generate(time_t set_time, long duration, std::string source, std::string reason, std::string xline_specific_mask)
+	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
 	{
 		return new SVSHold(set_time, duration, source, reason, xline_specific_mask);
 	}
 
-	bool AutoApplyToUserList(XLine *x)
+	bool AutoApplyToUserList(XLine* x) CXX11_OVERRIDE
 	{
 		return false;
 	}
@@ -103,16 +96,15 @@ class CommandSvshold : public Command
  public:
 	CommandSvshold(Module* Creator) : Command(Creator, "SVSHOLD", 1)
 	{
-		flags_needed = 'o'; this->syntax = "<nickname> [<duration> :<reason>]";
-		TRANSLATE4(TR_TEXT, TR_TEXT, TR_TEXT, TR_END);
+		flags_needed = 'o'; this->syntax = "<nick> [<duration> :<reason>]";
 	}
 
-	CmdResult Handle(const std::vector<std::string> &parameters, User *user)
+	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE
 	{
 		/* syntax: svshold nickname time :reason goes here */
 		/* 'time' is a human-readable timestring, like 2d3h2s. */
 
-		if (!ServerInstance->ULine(user->server))
+		if (!user->server->IsULine())
 		{
 			/* don't allow SVSHOLD from non-ulined clients */
 			return CMD_FAILURE;
@@ -120,14 +112,16 @@ class CommandSvshold : public Command
 
 		if (parameters.size() == 1)
 		{
-			if (ServerInstance->XLines->DelLine(parameters[0].c_str(), "SVSHOLD", user))
+			std::string reason;
+
+			if (ServerInstance->XLines->DelLine(parameters[0].c_str(), "SVSHOLD", reason, user))
 			{
 				if (!silent)
-					ServerInstance->SNO->WriteToSnoMask('x',"%s removed SVSHOLD on %s",user->nick.c_str(),parameters[0].c_str());
+					ServerInstance->SNO->WriteToSnoMask('x', "%s removed SVSHOLD on %s: %s", user->nick.c_str(), parameters[0].c_str(), reason.c_str());
 			}
 			else
 			{
-				user->WriteServ("NOTICE %s :*** SVSHOLD %s not found in list, try /stats S.",user->nick.c_str(),parameters[0].c_str());
+				user->WriteNotice("*** SVSHOLD " + parameters[0] + " not found on the list.");
 			}
 		}
 		else
@@ -135,8 +129,12 @@ class CommandSvshold : public Command
 			if (parameters.size() < 3)
 				return CMD_FAILURE;
 
-			// Adding - XXX todo make this respect <insane> tag perhaps..
-			long duration = ServerInstance->Duration(parameters[1]);
+			unsigned long duration;
+			if (!InspIRCd::Duration(parameters[1], duration))
+			{
+				user->WriteNotice("*** Invalid duration for SVSHOLD.");
+				return CMD_FAILURE;
+			}
 			SVSHold* r = new SVSHold(ServerInstance->Time(), duration, user->nick.c_str(), parameters[2].c_str(), parameters[0].c_str());
 
 			if (ServerInstance->XLines->AddLine(r, user))
@@ -150,9 +148,9 @@ class CommandSvshold : public Command
 				}
 				else
 				{
-					time_t c_requires_crap = duration + ServerInstance->Time();
-					std::string timestr = ServerInstance->TimeString(c_requires_crap);
-					ServerInstance->SNO->WriteGlobalSno('x', "%s added timed SVSHOLD for %s, expires on %s: %s", user->nick.c_str(), parameters[0].c_str(), timestr.c_str(), parameters[2].c_str());
+					ServerInstance->SNO->WriteGlobalSno('x', "%s added timed SVSHOLD for %s, expires in %s (on %s): %s",
+						user->nick.c_str(), parameters[0].c_str(), InspIRCd::DurationString(duration).c_str(),
+						InspIRCd::TimeString(ServerInstance->Time() + duration).c_str(), parameters[2].c_str());
 				}
 			}
 			else
@@ -165,69 +163,67 @@ class CommandSvshold : public Command
 		return CMD_SUCCESS;
 	}
 
-	RouteDescriptor GetRouting(User* user, const std::vector<std::string>& parameters)
+	RouteDescriptor GetRouting(User* user, const Params& parameters) CXX11_OVERRIDE
 	{
 		return ROUTE_BROADCAST;
 	}
 };
 
-class ModuleSVSHold : public Module
+class ModuleSVSHold : public Module, public Stats::EventListener
 {
 	CommandSvshold cmd;
 	SVSHoldFactory s;
 
 
  public:
-	ModuleSVSHold() : cmd(this)
+	ModuleSVSHold()
+		: Stats::EventListener(this)
+		, cmd(this)
 	{
 	}
 
-	void init()
+	void init() CXX11_OVERRIDE
 	{
 		ServerInstance->XLines->RegisterFactory(&s);
-		ServerInstance->Modules->AddService(cmd);
-		Implementation eventlist[] = { I_OnUserPreNick, I_OnStats, I_OnRehash };
-		ServerInstance->Modules->Attach(eventlist, this, sizeof(eventlist)/sizeof(Implementation));
-		OnRehash(NULL);
 	}
 
-	void OnRehash(User* user)
+	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
 	{
 		ConfigTag* tag = ServerInstance->Config->ConfValue("svshold");
-		silent = tag->getBool("silent");
+		silent = tag->getBool("silent", true);
 	}
 
-	virtual ModResult OnStats(char symbol, User* user, string_list &out)
+	ModResult OnStats(Stats::Context& stats) CXX11_OVERRIDE
 	{
-		if(symbol != 'S')
+		if (stats.GetSymbol() != 'S')
 			return MOD_RES_PASSTHRU;
 
-		ServerInstance->XLines->InvokeStats("SVSHOLD", 210, user, out);
+		ServerInstance->XLines->InvokeStats("SVSHOLD", 210, stats);
 		return MOD_RES_DENY;
 	}
 
-	virtual ModResult OnUserPreNick(User *user, const std::string &newnick)
+	ModResult OnUserPreNick(LocalUser* user, const std::string& newnick) CXX11_OVERRIDE
 	{
 		XLine *rl = ServerInstance->XLines->MatchesLine("SVSHOLD", newnick);
 
 		if (rl)
 		{
-			user->WriteServ( "432 %s %s :Services reserved nickname: %s", user->nick.c_str(), newnick.c_str(), rl->reason.c_str());
+			user->WriteNumeric(ERR_ERRONEUSNICKNAME, newnick, InspIRCd::Format("Services reserved nickname: %s", rl->reason.c_str()));
 			return MOD_RES_DENY;
 		}
 
 		return MOD_RES_PASSTHRU;
 	}
 
-	virtual ~ModuleSVSHold()
+	~ModuleSVSHold()
 	{
 		ServerInstance->XLines->DelAll("SVSHOLD");
 		ServerInstance->XLines->UnregisterFactory(&s);
 	}
 
-	virtual Version GetVersion()
+	Version GetVersion() CXX11_OVERRIDE
 	{
-		return Version("Implements SVSHOLD. Like Q:Lines, but can only be added/removed by Services.", VF_COMMON | VF_VENDOR);
+		return Version("Implements SVSHOLD, like Q-lines, but can only be added/removed by Services", VF_COMMON | VF_VENDOR);
 	}
 };
 
