@@ -20,40 +20,37 @@
 
 
 #include "inspircd.h"
-#include "m_cap.h"
+#include "modules/cap.h"
+#include "modules/names.h"
 
-/* $ModDesc: Provides the UHNAMES facility. */
-
-class ModuleUHNames : public Module
+class ModuleUHNames
+	: public Module
+	, public Names::EventListener
 {
+ private:
+	Cap::Capability cap;
+
  public:
-	GenericCap cap;
-
-	ModuleUHNames() : cap(this, "userhost-in-names")
+	ModuleUHNames()
+		: Names::EventListener(this)
+		, cap(this, "userhost-in-names")
 	{
 	}
 
-	void init()
+	Version GetVersion() CXX11_OVERRIDE
 	{
-		Implementation eventlist[] = { I_OnEvent, I_OnPreCommand, I_OnNamesListItem, I_On005Numeric };
-		ServerInstance->Modules->Attach(eventlist, this, sizeof(eventlist)/sizeof(Implementation));
+		return Version("Provides the UHNAMES (CAP userhost-in-names) capability", VF_VENDOR);
 	}
 
-	~ModuleUHNames()
+	void On005Numeric(std::map<std::string, std::string>& tokens) CXX11_OVERRIDE
 	{
+		// The legacy PROTOCTL system is a wrapper around the cap.
+		dynamic_reference_nocheck<Cap::Manager> capmanager(this, "capmanager");
+		if (capmanager)
+			tokens["UHNAMES"];
 	}
 
-	Version GetVersion()
-	{
-		return Version("Provides the UHNAMES facility.",VF_VENDOR);
-	}
-
-	void On005Numeric(std::string &output)
-	{
-		output.append(" UHNAMES");
-	}
-
-	ModResult OnPreCommand(std::string &command, std::vector<std::string> &parameters, LocalUser *user, bool validated, const std::string &original_line)
+	ModResult OnPreCommand(std::string& command, CommandBase::Params& parameters, LocalUser* user, bool validated) CXX11_OVERRIDE
 	{
 		/* We don't actually create a proper command handler class for PROTOCTL,
 		 * because other modules might want to have PROTOCTL hooks too.
@@ -64,27 +61,19 @@ class ModuleUHNames : public Module
 		{
 			if ((parameters.size()) && (!strcasecmp(parameters[0].c_str(),"UHNAMES")))
 			{
-				cap.ext.set(user, 1);
+				cap.set(user, true);
 				return MOD_RES_DENY;
 			}
 		}
 		return MOD_RES_PASSTHRU;
 	}
 
-	void OnNamesListItem(User* issuer, Membership* memb, std::string &prefixes, std::string &nick)
+	ModResult OnNamesListItem(LocalUser* issuer, Membership* memb, std::string& prefixes, std::string& nick) CXX11_OVERRIDE
 	{
-		if (!cap.ext.get(issuer))
-			return;
+		if (cap.get(issuer))
+			nick = memb->user->GetFullHost();
 
-		if (nick.empty())
-			return;
-
-		nick = memb->user->GetFullHost();
-	}
-
-	void OnEvent(Event& ev)
-	{
-		cap.HandleEvent(ev);
+		return MOD_RES_PASSTHRU;
 	}
 };
 
