@@ -17,14 +17,7 @@
  */
 
 
-/* $Core */
-
-/*********        DEFAULTS       **********/
-/* $ExtraSources: threadengines/threadengine_pthread.cpp */
-/* $ExtraObjects: threadengine_pthread.o */
-
 #include "inspircd.h"
-#include "threadengine.h"
 
 void Thread::SetExitFlag()
 {
@@ -33,14 +26,5 @@ void Thread::SetExitFlag()
 
 void Thread::join()
 {
-		state->FreeThread(this);
-		delete state;
-		state = 0;
-}
-
-/** If this thread has a Creator set, call it to
- * free the thread
- */
-Thread::~Thread()
-{
+	ServerInstance->Threads.Stop(this);
 }
