@@ -23,7 +23,7 @@
 
 #include "inspircd.h"
 #include "xline.h"
-#include "bancache.h"
+#include "modules/stats.h"
 
 /** An XLineFactory specialized to generate GLine* pointers
  */
@@ -34,7 +34,7 @@ class GLineFactory : public XLineFactory
 
 	/** Generate a GLine
 	 */
-	XLine* Generate(time_t set_time, long duration, std::string source, std::string reason, std::string xline_specific_mask)
+	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
 	{
 		IdentHostPair ih = ServerInstance->XLines->IdentSplit(xline_specific_mask);
 		return new GLine(set_time, duration, source, reason, ih.first, ih.second);
@@ -50,7 +50,7 @@ class ELineFactory : public XLineFactory
 
 	/** Generate an ELine
 	 */
-	XLine* Generate(time_t set_time, long duration, std::string source, std::string reason, std::string xline_specific_mask)
+	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
 	{
 		IdentHostPair ih = ServerInstance->XLines->IdentSplit(xline_specific_mask);
 		return new ELine(set_time, duration, source, reason, ih.first, ih.second);
@@ -66,7 +66,7 @@ class KLineFactory : public XLineFactory
 
 	/** Generate a KLine
 	 */
-        XLine* Generate(time_t set_time, long duration, std::string source, std::string reason, std::string xline_specific_mask)
+        XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
         {
                 IdentHostPair ih = ServerInstance->XLines->IdentSplit(xline_specific_mask);
                 return new KLine(set_time, duration, source, reason, ih.first, ih.second);
@@ -82,7 +82,7 @@ class QLineFactory : public XLineFactory
 
 	/** Generate a QLine
 	 */
-        XLine* Generate(time_t set_time, long duration, std::string source, std::string reason, std::string xline_specific_mask)
+        XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
         {
                 return new QLine(set_time, duration, source, reason, xline_specific_mask);
         }
@@ -97,7 +97,7 @@ class ZLineFactory : public XLineFactory
 
 	/** Generate a ZLine
 	 */
-        XLine* Generate(time_t set_time, long duration, std::string source, std::string reason, std::string xline_specific_mask)
+        XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
         {
                 return new ZLine(set_time, duration, source, reason, xline_specific_mask);
         }
@@ -156,9 +156,10 @@ void XLineManager::CheckELines()
 	if (ELines.empty())
 		return;
 
-	for (LocalUserList::const_iterator u2 = ServerInstance->Users->local_users.begin(); u2 != ServerInstance->Users->local_users.end(); u2++)
+	const UserManager::LocalList& list = ServerInstance->Users.GetLocalUsers();
+	for (UserManager::LocalList::const_iterator u2 = list.begin(); u2 != list.end(); u2++)
 	{
-		User* u = (User*)(*u2);
+		LocalUser* u = *u2;
 		u->exempt = false;
 
 		/* This uses safe iteration to ensure that if a line expires here, it doenst trash the iterator */
@@ -264,12 +265,31 @@ bool XLineManager::AddLine(XLine* line, User* user)
 		LookupIter i = x->second.find(line->Displayable());
 		if (i != x->second.end())
 		{
-			// XLine propagation bug was here, if the line to be added already exists and
-			// it's expired then expire it and add the new one instead of returning false
-			if ((!i->second->duration) || (ServerInstance->Time() < i->second->expiry))
-				return false;
+			bool silent = false;
 
-			ExpireLine(x, i);
+			// Allow replacing a config line for an updated config line.
+			if (i->second->from_config && line->from_config)
+			{
+				// Nothing changed, skip adding this one.
+				if (i->second->reason == line->reason)
+					return false;
+
+				silent = true;
+			}
+			// Allow replacing a non-config line for a new config line.
+			else if (!line->from_config)
+			{
+				// X-line propagation bug was here, if the line to be added already exists and
+				// it's expired then expire it and add the new one instead of returning false
+				if ((!i->second->duration) || (ServerInstance->Time() < i->second->expiry))
+					return false;
+			}
+			else
+			{
+				silent = true;
+			}
+
+			ExpireLine(x, i, silent);
 		}
 	}
 
@@ -278,7 +298,7 @@ bool XLineManager::AddLine(XLine* line, User* user)
 	if (!xlf)
 		return false;
 
-	ServerInstance->BanCache->RemoveEntries(line->type, false); // XXX perhaps remove ELines here?
+	ServerInstance->BanCache.RemoveEntries(line->type, false); // XXX perhaps remove ELines here?
 
 	if (xlf->AutoApplyToUserList(line))
 		pending_lines.push_back(line);
@@ -286,14 +306,14 @@ bool XLineManager::AddLine(XLine* line, User* user)
 	lookup_lines[line->type][line->Displayable()] = line;
 	line->OnAdd();
 
-	FOREACH_MOD(I_OnAddLine,OnAddLine(user, line));
+	FOREACH_MOD(OnAddLine, (user, line));
 
 	return true;
 }
 
 // deletes a line, returns true if the line existed and was removed
 
-bool XLineManager::DelLine(const char* hostmask, const std::string &type, User* user, bool simulate)
+bool XLineManager::DelLine(const char* hostmask, const std::string& type, std::string& reason, User* user, bool simulate)
 {
 	ContainerIter x = lookup_lines.find(type);
 
@@ -305,18 +325,18 @@ bool XLineManager::DelLine(const char* hostmask, const std::string &type, User*
 	if (y == x->second.end())
 		return false;
 
+	reason.assign(y->second->reason);
+
 	if (simulate)
 		return true;
 
-	ServerInstance->BanCache->RemoveEntries(y->second->type, true);
+	ServerInstance->BanCache.RemoveEntries(y->second->type, true);
 
-	FOREACH_MOD(I_OnDelLine,OnDelLine(user, y->second));
+	FOREACH_MOD(OnDelLine, (user, y->second));
 
 	y->second->Unset();
 
-	std::vector<XLine*>::iterator pptr = std::find(pending_lines.begin(), pending_lines.end(), y->second);
-	if (pptr != pending_lines.end())
-		pending_lines.erase(pptr);
+	stdalgo::erase(pending_lines, y->second);
 
 	delete y->second;
 	x->second.erase(y);
@@ -330,7 +350,7 @@ void ELine::Unset()
 	ServerInstance->XLines->CheckELines();
 }
 
-// returns a pointer to the reason if a nickname matches a qline, NULL if it didnt match
+// returns a pointer to the reason if a nickname matches a Q-line, NULL if it didn't match
 
 XLine* XLineManager::MatchesLine(const std::string &type, User* user)
 {
@@ -402,20 +422,20 @@ XLine* XLineManager::MatchesLine(const std::string &type, const std::string &pat
 }
 
 // removes lines that have expired
-void XLineManager::ExpireLine(ContainerIter container, LookupIter item)
+void XLineManager::ExpireLine(ContainerIter container, LookupIter item, bool silent)
 {
-	FOREACH_MOD(I_OnExpireLine, OnExpireLine(item->second));
+	FOREACH_MOD(OnExpireLine, (item->second));
+
+	if (!silent)
+		item->second->DisplayExpiry();
 
-	item->second->DisplayExpiry();
 	item->second->Unset();
 
 	/* TODO: Can we skip this loop by having a 'pending' field in the XLine class, which is set when a line
 	 * is pending, cleared when it is no longer pending, so we skip over this loop if its not pending?
 	 * -- Brain
 	 */
-	std::vector<XLine*>::iterator pptr = std::find(pending_lines.begin(), pending_lines.end(), item->second);
-	if (pptr != pending_lines.end())
-		pending_lines.erase(pptr);
+	stdalgo::erase(pending_lines, item->second);
 
 	delete item->second;
 	container->second.erase(item);
@@ -425,10 +445,10 @@ void XLineManager::ExpireLine(ContainerIter container, LookupIter item)
 // applies lines, removing clients and changing nicks etc as applicable
 void XLineManager::ApplyLines()
 {
-	LocalUserList::reverse_iterator u2 = ServerInstance->Users->local_users.rbegin();
-	while (u2 != ServerInstance->Users->local_users.rend())
+	const UserManager::LocalList& list = ServerInstance->Users.GetLocalUsers();
+	for (UserManager::LocalList::const_iterator j = list.begin(); j != list.end(); )
 	{
-		User* u = *u2++;
+		LocalUser* u = *j++;
 
 		// Don't ban people who are exempt.
 		if (u->exempt)
@@ -438,14 +458,21 @@ void XLineManager::ApplyLines()
 		{
 			XLine *x = *i;
 			if (x->Matches(u))
+			{
 				x->Apply(u);
+
+				// If applying the X-line has killed the user then don't
+				// apply any more lines to them.
+				if (u->quitting)
+					break;
+			}
 		}
 	}
 
 	pending_lines.clear();
 }
 
-void XLineManager::InvokeStats(const std::string &type, int numeric, User* user, string_list &results)
+void XLineManager::InvokeStats(const std::string& type, unsigned int numeric, Stats::Context& stats)
 {
 	ContainerIter n = lookup_lines.find(type);
 
@@ -466,7 +493,7 @@ void XLineManager::InvokeStats(const std::string &type, int numeric, User* user,
 				ExpireLine(n, i);
 			}
 			else
-				results.push_back(ServerInstance->Config->ServerName+" "+ConvToStr(numeric)+" "+user->nick+" :"+i->second->Displayable()+" "+
+				stats.AddRow(numeric, i->second->Displayable()+" "+
 					ConvToStr(i->second->set_time)+" "+ConvToStr(i->second->duration)+" "+i->second->source+" :"+i->second->reason);
 			i = safei;
 		}
@@ -521,40 +548,38 @@ void XLine::Apply(User* u)
 
 bool XLine::IsBurstable()
 {
-	return true;
+	return !from_config;
 }
 
 void XLine::DefaultApply(User* u, const std::string &line, bool bancache)
 {
-	char sreason[MAXBUF];
-	snprintf(sreason, MAXBUF, "%s-Lined: %s", line.c_str(), this->reason.c_str());
-	if (!ServerInstance->Config->MoronBanner.empty())
-		u->WriteServ("NOTICE %s :*** %s", u->nick.c_str(), ServerInstance->Config->MoronBanner.c_str());
+	const std::string banReason = line + "-lined: " + reason;
+
+	if (!ServerInstance->Config->XLineMessage.empty())
+		u->WriteNumeric(ERR_YOUREBANNEDCREEP, ServerInstance->Config->XLineMessage);
 
 	if (ServerInstance->Config->HideBans)
-		ServerInstance->Users->QuitUser(u, line + "-Lined", sreason);
+		ServerInstance->Users->QuitUser(u, line + "-lined", &banReason);
 	else
-		ServerInstance->Users->QuitUser(u, sreason);
+		ServerInstance->Users->QuitUser(u, banReason);
 
 
 	if (bancache)
 	{
-		ServerInstance->Logs->Log("BANCACHE", DEBUG, "BanCache: Adding positive hit (" + line + ") for " + u->GetIPString());
-		if (this->duration > 0)
-			ServerInstance->BanCache->AddHit(u->GetIPString(), this->type, line + "-Lined: " + this->reason, this->duration);
-		else
-			ServerInstance->BanCache->AddHit(u->GetIPString(), this->type, line + "-Lined: " + this->reason);
+		ServerInstance->Logs->Log("BANCACHE", LOG_DEBUG, "BanCache: Adding positive hit (" + line + ") for " + u->GetIPString());
+		ServerInstance->BanCache.AddHit(u->GetIPString(), this->type, banReason, (this->duration > 0 ? (this->expiry - ServerInstance->Time()) : 0));
 	}
 }
 
 bool KLine::Matches(User *u)
 {
-	if (u->exempt)
+	LocalUser* lu = IS_LOCAL(u);
+	if (lu && lu->exempt)
 		return false;
 
 	if (InspIRCd::Match(u->ident, this->identmask, ascii_case_insensitive_map))
 	{
-		if (InspIRCd::MatchCIDR(u->host, this->hostmask, ascii_case_insensitive_map) ||
+		if (InspIRCd::MatchCIDR(u->GetRealHost(), this->hostmask, ascii_case_insensitive_map) ||
 		    InspIRCd::MatchCIDR(u->GetIPString(), this->hostmask, ascii_case_insensitive_map))
 		{
 			return true;
@@ -571,12 +596,13 @@ void KLine::Apply(User* u)
 
 bool GLine::Matches(User *u)
 {
-	if (u->exempt)
+	LocalUser* lu = IS_LOCAL(u);
+	if (lu && lu->exempt)
 		return false;
 
 	if (InspIRCd::Match(u->ident, this->identmask, ascii_case_insensitive_map))
 	{
-		if (InspIRCd::MatchCIDR(u->host, this->hostmask, ascii_case_insensitive_map) ||
+		if (InspIRCd::MatchCIDR(u->GetRealHost(), this->hostmask, ascii_case_insensitive_map) ||
 		    InspIRCd::MatchCIDR(u->GetIPString(), this->hostmask, ascii_case_insensitive_map))
 		{
 			return true;
@@ -595,7 +621,7 @@ bool ELine::Matches(User *u)
 {
 	if (InspIRCd::Match(u->ident, this->identmask, ascii_case_insensitive_map))
 	{
-		if (InspIRCd::MatchCIDR(u->host, this->hostmask, ascii_case_insensitive_map) ||
+		if (InspIRCd::MatchCIDR(u->GetRealHost(), this->hostmask, ascii_case_insensitive_map) ||
 		    InspIRCd::MatchCIDR(u->GetIPString(), this->hostmask, ascii_case_insensitive_map))
 		{
 			return true;
@@ -607,7 +633,8 @@ bool ELine::Matches(User *u)
 
 bool ZLine::Matches(User *u)
 {
-	if (u->exempt)
+	LocalUser* lu = IS_LOCAL(u);
+	if (lu && lu->exempt)
 		return false;
 
 	if (InspIRCd::MatchCIDR(u->GetIPString(), this->ipaddr))
@@ -632,8 +659,8 @@ bool QLine::Matches(User *u)
 
 void QLine::Apply(User* u)
 {
-	/* Force to uuid on apply of qline, no need to disconnect any more :) */
-	u->ForceNickChange(u->uuid.c_str());
+	/* Force to uuid on apply of Q-line, no need to disconnect anymore :) */
+	u->ChangeNick(u->uuid);
 }
 
 
@@ -670,68 +697,46 @@ bool GLine::Matches(const std::string &str)
 
 void ELine::OnAdd()
 {
-	/* When adding one eline, only check the one eline */
-	for (LocalUserList::const_iterator u2 = ServerInstance->Users->local_users.begin(); u2 != ServerInstance->Users->local_users.end(); u2++)
+	/* When adding one E-line, only check the one E-line */
+	const UserManager::LocalList& list = ServerInstance->Users.GetLocalUsers();
+	for (UserManager::LocalList::const_iterator u2 = list.begin(); u2 != list.end(); u2++)
 	{
-		User* u = (User*)(*u2);
+		LocalUser* u = *u2;
 		if (this->Matches(u))
 			u->exempt = true;
 	}
 }
 
-void ELine::DisplayExpiry()
+void XLine::DisplayExpiry()
 {
-	ServerInstance->SNO->WriteToSnoMask('x',"Removing expired E-Line %s@%s (set by %s %ld seconds ago)",
-		identmask.c_str(),hostmask.c_str(),source.c_str(),(long)(ServerInstance->Time() - this->set_time));
+	bool onechar = (type.length() == 1);
+	ServerInstance->SNO->WriteToSnoMask('x', "Removing expired %s%s %s (set by %s %s ago): %s",
+		type.c_str(), (onechar ? "-line" : ""), Displayable().c_str(), source.c_str(), InspIRCd::DurationString(ServerInstance->Time() - set_time).c_str(), reason.c_str());
 }
 
-void QLine::DisplayExpiry()
+const std::string& ELine::Displayable()
 {
-	ServerInstance->SNO->WriteToSnoMask('x',"Removing expired Q-Line %s (set by %s %ld seconds ago)",
-		nick.c_str(),source.c_str(),(long)(ServerInstance->Time() - this->set_time));
+	return matchtext;
 }
 
-void ZLine::DisplayExpiry()
+const std::string& KLine::Displayable()
 {
-	ServerInstance->SNO->WriteToSnoMask('x',"Removing expired Z-Line %s (set by %s %ld seconds ago)",
-		ipaddr.c_str(),source.c_str(),(long)(ServerInstance->Time() - this->set_time));
+	return matchtext;
 }
 
-void KLine::DisplayExpiry()
+const std::string& GLine::Displayable()
 {
-	ServerInstance->SNO->WriteToSnoMask('x',"Removing expired K-Line %s@%s (set by %s %ld seconds ago)",
-		identmask.c_str(),hostmask.c_str(),source.c_str(),(long)(ServerInstance->Time() - this->set_time));
+	return matchtext;
 }
 
-void GLine::DisplayExpiry()
+const std::string& ZLine::Displayable()
 {
-	ServerInstance->SNO->WriteToSnoMask('x',"Removing expired G-Line %s@%s (set by %s %ld seconds ago)",
-		identmask.c_str(),hostmask.c_str(),source.c_str(),(long)(ServerInstance->Time() - this->set_time));
+	return ipaddr;
 }
 
-const char* ELine::Displayable()
+const std::string& QLine::Displayable()
 {
-	return matchtext.c_str();
-}
-
-const char* KLine::Displayable()
-{
-	return matchtext.c_str();
-}
-
-const char* GLine::Displayable()
-{
-	return matchtext.c_str();
-}
-
-const char* ZLine::Displayable()
-{
-	return ipaddr.c_str();
-}
-
-const char* QLine::Displayable()
-{
-	return nick.c_str();
+	return nick;
 }
 
 bool KLine::IsBurstable()
@@ -772,3 +777,24 @@ XLineFactory* XLineManager::GetFactory(const std::string &type)
 
 	return n->second;
 }
+
+void XLineManager::ExpireRemovedConfigLines(const std::string& type, const insp::flat_set<std::string>& configlines)
+{
+	// Nothing to do.
+	if (lookup_lines.empty())
+		return;
+
+	ContainerIter xlines = lookup_lines.find(type);
+	if (xlines == lookup_lines.end())
+		return;
+
+	for (LookupIter xline = xlines->second.begin(); xline != xlines->second.end(); )
+	{
+		LookupIter cachedxline = xline++;
+		if (!cachedxline->second->from_config)
+			continue;
+
+		if (!configlines.count(cachedxline->second->Displayable()))
+			ExpireLine(xlines, cachedxline);
+	}
+}
